@@ -1,10 +1,10 @@
-/* $Id: board.cc,v 1.94 2003/08/17 23:10:09 bergo Exp $ */
+/* $Id: board.cc,v 1.121 2008/02/08 14:25:50 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -33,13 +33,22 @@
 #include "global.h"
 #include "text.h"
 #include "protocol.h"
+#include "dgtboard.h"
 
 #include "mainwindow.h"
 
 Board * Board::PopupOwner = 0;
 
+static GdkEventMotion LastMotionEvent;
+
 // --- Board --------------------------
 
+#if MAEMO
+void tap_and_hold_cb(GtkWidget *widget, gpointer user_data) {
+  ((Board*)user_data)->popupProtocolMenu(100,100,0);
+}
+#endif
+
 Board::Board() : WidgetProxy() {
   int i;
 
@@ -62,6 +71,13 @@ Board::Board() : WidgetProxy() {
   dr_active=false;
   FlipInvert=false;
   EditMode=false;
+  UpdateClockAfterConfigure = false;
+  jx=jy=-1;
+  jpxv = 0; jpyv = 0;
+
+  jsx=jsy=-1;
+  jsvx=jsvy=0;
+  jstid=-1;
 
   AnimationTimeout = -1;
 
@@ -73,18 +89,9 @@ Board::Board() : WidgetProxy() {
   repaint_due=false;
   frozen_lvl=0;
 
-  f1=global.loadFont(EF_PlayerFont);
-  f2=global.loadFont(EF_ClockFont);
-  f3=global.loadFont(EF_InfoFont);
-
   if (global.ShowCoordinates) { borx=20; bory=20; } else { borx=bory=0; }
   morey=0;
 
-  if ((!f1)||(!f2)||(!f3)) {
-    cerr << _("<Board::Board> ** failed to load one or more board fonts - install X11 75 dpi and 100 dpi fonts, restart X, and try again.\n");
-    exit(2);
-  }
-
   global.BoardList.push_back(this);
 
   canselect=true;
@@ -124,6 +131,20 @@ Board::Board() : WidgetProxy() {
                       (GtkSignalFunc) board_motion_event,
 		      (gpointer) this);
 
+#if MAEMO
+  // Note: the following code does not work
+  // I get no events :-/
+  gtk_widget_tap_and_hold_setup (yidget, NULL, NULL, GtkWidgetTapAndHoldFlags(0));
+#if 0
+  gtk_signal_connect (GTK_OBJECT (yidget), "tap-and-hold",
+		      //(GtkSignalFunc) board_button_press_event,
+		      (GtkSignalFunc) tap_and_hold_cb ,
+		      (gpointer) this);
+#else
+  g_signal_connect (G_OBJECT (yidget), "tap-and-hold", G_CALLBACK (tap_and_hold_cb), (gpointer) this);
+#endif
+#endif
+
   gshow(widget);
 }
 
@@ -134,21 +155,26 @@ bool Board::hasGame() {
 Board::~Board() {
   unsigned int i;
   global.debug("Board","~Board");
+
+  if (mygame!=NULL)
+    if (mygame->getBoard() == this)
+      mygame->setBoard(NULL);
+
   global.removeBoard(this);
   global.removePieceClient(this);
-  if (f1)
-    gdk_font_unref(f1);
-  if (f2)
-    gdk_font_unref(f2);
-  if (f3)
-    gdk_font_unref(f3);
   for(i=0;i<LastMove.size();i++)
     delete(LastMove[i]);
   LastMove.clear();
-  if (cur)
+  if (cur) {
     delete cur;
+    cur = NULL;
+  }
   if (wgc)
     gdk_gc_destroy(wgc);
+  if (scrap!=NULL) {
+    g_free(scrap);
+    scrap = NULL;
+  }    
   gdk_gc_destroy(clkgc);
   gdk_pixmap_unref(clkbar);
   gdk_pixmap_unref(buffer);
@@ -157,18 +183,6 @@ Board::~Board() {
 }
 
 void Board::reloadFonts() {
-  if (f1) gdk_font_unref(f1);
-  if (f2) gdk_font_unref(f2);
-  if (f3) gdk_font_unref(f3);
-
-  f1=global.loadFont(EF_PlayerFont);
-  f2=global.loadFont(EF_ClockFont);
-  f3=global.loadFont(EF_InfoFont);
-
-  if ((!f1)||(!f2)||(!f3)) {
-    cerr << _("<Board::Board> ** failed to load one or more board fonts\n");
-    exit(2);
-  }
   clock_ch=1;
   queueRepaint();
 }
@@ -211,7 +225,7 @@ void Board::reset() {
   for(i=0;i<6;i++)
     info[i].erase();
   clock.setMirror(0);
-  clock.setClock(0,0,0,1);
+  clock.setClock2(0,0,0,1);
   currently_rendered.invalidate();
   position.setStartPos();
   for(i=0;i<LastMove.size();i++)
@@ -240,6 +254,7 @@ void Board::pieceSetChanged() {
   currently_rendered.invalidate();
   if (global.ShowCoordinates) { borx=20; bory=20; } else { borx=bory=0; }
   morey=0;
+  UpdateClockAfterConfigure = true;
   queueRepaint();
 }
 
@@ -310,11 +325,14 @@ void Board::updateClock() {
 		    clkbar,
 		    0, 0,
 		    Width(), 0,250, Height());
+    if (dr_active)
+      board_motion_event(widget,&LastMotionEvent,(gpointer) this);
   }
 }
 
-void RootBoard::clockString(int val,char *z) {
-  int cv,neg,H,M,S;
+void RootBoard::clockString(int val,char *z,int maxlen) {
+  static char *minus = "-";
+  int cv,mr,neg,H,M,S;
   cv=val;
   if (cv<0) {
     cv=-cv;
@@ -322,20 +340,23 @@ void RootBoard::clockString(int val,char *z) {
   } else
     neg=0;
   
+  mr = cv%1000;
+  cv /= 1000;
+
   H=cv/3600;
   M=(cv-3600*H)/60;
   S=(cv-3600*H)%60;
 
   if (H) {
-    if (neg)
-      sprintf(z,"-%d:%.2d:%.2d",H,M,S);
-    else
-      sprintf(z,"%d:%.2d:%.2d",H,M,S);
+    snprintf(z,maxlen,"%s%d:%.2d:%.2d",
+	     neg?minus:&minus[1],H,M,S);
   } else {
-    if (neg)
-      sprintf(z,"-%.2d:%.2d",M,S);
+    if (mr && M==0)
+      snprintf(z,maxlen,"%s%.2d:%.2d.%.3d",
+	       neg?minus:&minus[1],M,S,mr);
     else
-      sprintf(z,"%.2d:%.2d",M,S);
+      snprintf(z,maxlen,"%s%.2d:%.2d",
+	       neg?minus:&minus[1],M,S);
   }
 }
 
@@ -359,62 +380,67 @@ void Board::renderClock() {
 
   sq=sqside;
 
-  C.prepare(clkbar,gc,0,0,250,sqh);
+  C.prepare(widget,clkbar,gc,0,0,250,sqh);
   C.setColor(0);
   C.drawRect(0,0,C.W,C.H,true);
   C.H = Height();
 
-  bh=56; ch=20; th=36; // font heights
-  C.setFont(0,f1);
-  C.setFont(1,f2);
-  C.setFont(2,f3);
+  C.setFont(0,global.PlayerFont);
+  C.setFont(1,global.ClockFont);
+  C.setFont(2,global.InfoFont);
+
+  ch = C.fontHeight(1);
+  th = C.fontHeight(0);
+  bh = ch + th;
 
   if (sq*3 < bh) {
-    bh=28; ch=14; th=14;
-    C.setFont(0,f3);
-    C.setFont(1,f3);
-    C.setFont(2,f3);
+    C.setFont(0,global.InfoFont);
+    C.setFont(1,global.InfoFont);
+    C.setFont(2,global.InfoFont);
+    ch = C.fontHeight(1);
+    th = C.fontHeight(0);
+    bh = ch + th;
   }
 
   // draw the clock time and White/Black labels
 
   ef=effectiveFlip();
-  if (ef) i= C.H - (th+10); else i = ch+10;
+  if (ef) i= C.H - bh; else i = 0;
   j=16;
 
   C.setColor(0xffffff);
   if (clock.getActive()>0) {
     if (lowtime) C.setColor(0xff8080);
-    C.drawRect(15, i - (ch+2), 200-30, bh+4, true);
+    C.drawRect(15, i, 200-30, bh+4, true);
     C.setColor(0);
   }
 
   C.drawString(j,i,0,_("Black"));
-  clockString(clock.getValue(1),z);
+  clockString(clock.getValue2(1),z,64);
   C.drawString(j,i+th,1,z);
 
-  if (ef) i=ch+10; else i= C.H - (th+10);
+  if (ef) i=0; else i= C.H - bh;
 
   C.setColor(0xffffff);
   if (clock.getActive()<0) {
     if (lowtime) C.setColor(0xff8080);
-    C.drawRect(15, i - (ch+2), 200-30, bh+4, true);
+    C.drawRect(15, i, 200-30, bh+4, true);
     C.setColor(0);
   }
   C.drawString(j,i,0,_("White"));
-  clockString(clock.getValue(0),z);
+  clockString(clock.getValue2(0),z,64);
   C.drawString(j,i+th,1,z);
 
-  C.consumeTop(bh+20);
-  C.consumeBottom(bh+20);
+  C.consumeTop(bh+10);
+  C.consumeBottom(bh+10);
 
   // if we have enough space, render player names too
-  if ( (mygame) && (C.H > 60) ) {
+  if ( (mygame) && (C.H > 2*th) ) {
     C.setColor(0xffffff);
-    C.drawString(j,C.Y + ch,0, mygame->getPlayerString(ef?0:1) );
-    C.drawString(j,C.Y + C.H,0, mygame->getPlayerString(ef?1:0) );
-    C.consumeTop(ch+8);
-    C.consumeBottom(ch+8);
+    C.drawString(j,C.Y,0, mygame->getPlayerString(ef?0:1) );
+    C.drawString(j,C.Y + C.H - th,0, mygame->getPlayerString(ef?1:0) );
+    C.consumeTop(th+8);
+    C.consumeBottom(th+8);
   }
 
   cleanTargets();
@@ -426,16 +452,16 @@ void Board::renderClock() {
 
     for(k=0;k<5;k++) {
       if (C.H < 16) break;
-      C.drawString(j, C.Y + 16, 2, info[k] );
+      C.drawString(j, C.Y, 2, info[k] );
       C.consumeTop(16);
     }
     
     // house string is info[5]
 
     if (!info[5].empty()) {      
-      if ((!global.DrawHouseStock)||(C.H < 96)) {
+      if ((!global.DrawHouseStock)||(C.H < 96)||(EditMode)) {
 	if (C.H >= 16) {
-	  C.drawString(j,C.Y+16,2,info[5]);
+	  C.drawString(j,C.Y,2,info[5]);
 	  C.consumeTop(16);
 	}
       } else {
@@ -460,7 +486,7 @@ void Board::renderClock() {
 	}
 
 	// draw stock pieces
-	int eph, my;
+	int eph, my, fh2, phs, pcsw;
 
 	if (global.UseVectorPieces)
 	  eph=40+8;
@@ -468,13 +494,15 @@ void Board::renderClock() {
 	  eph=cur->side + 8;
 
 	if (ef) { we=0; be=eph; } else { we=eph; be=0; }
+	fh2 = C.fontHeight(2);
+	if (!global.UseVectorPieces)
+	  phs = cur->side/2;
 
 	my=2*eph;
 	C.setColor(0x505050);
 	for(si=2;si<my+2;si+=2)
 	  C.drawLine(5,C.Y+si,235,C.Y+si);
 
-
 	for(si=3;si<my+2;si+=2) {
 	  int lv;
 	  lv=my/2 - si;
@@ -498,42 +526,40 @@ void Board::renderClock() {
 	    if (global.UseVectorPieces)
 	      global.vpieces.drawPiece(clkbar,gc,40,40*si,C.Y+we,pk[si]|WHITE);
 	    else
-	      cur->drawPiece(pk[si]|WHITE,clkbar,gc,40*si, C.Y+we);
+	      cur->drawPiece(pk[si]|WHITE,clkbar,gc,40*si+20-phs, C.Y+we);
 
-	    sprintf(z,"%d",st[si][0]);
+	    snprintf(z,64,"%d",st[si][0]);
 
-	    shadyText(40*si+20+1, C.Y+eph+we, 2, z);
-
-	    /*
-	    C.setColor(0);
-	    C.drawString(40*si+20+1, C.Y+eph+we, 2, z);
-	    C.setColor(0xffffff);
-	    C.drawString(40*si+20, C.Y+eph+we, 2, z);
-	    */
+	    pcsw = C.stringWidth(2,z);
+	    shadyText(40*si+20-pcsw/2, C.Y+eph+we-fh2, 2, z);
 
 	    if (mygame->MyColor==WHITE)
 	      addTarget(new DropSource(pk[si]|WHITE,Width()+40*si,C.Y+we,40,eph));
+
+	    /*
+	    C.setColor(0xffff00);
+	    C.drawRect(40*si,C.Y+we,40,eph,false);
+	    */
 	  }
 	  if (st[si][1]) {
 
 	    if (global.UseVectorPieces)
 	      global.vpieces.drawPiece(clkbar,gc,40,40*si,C.Y+be,pk[si]|BLACK);
 	    else
-	      cur->drawPiece(pk[si]|BLACK,clkbar,gc,40*si, C.Y+be);
+	      cur->drawPiece(pk[si]|BLACK,clkbar,gc,40*si+20-phs, C.Y+be);
 
-	    sprintf(z,"%d",st[si][1]);
+	    snprintf(z,64,"%d",st[si][1]);
 
-	    shadyText(40*si+20+1, C.Y+eph+be, 2, z);
-
-	    /*
-	    C.setColor(0);
-	    C.drawString(40*si+20+1,C.Y+eph+be, 2, z);
-	    C.setColor(0xffffff);
-	    C.drawString(40*si+20,C.Y+eph+be, 2, z);
-	    */
+	    pcsw = C.stringWidth(2,z);
+	    shadyText(40*si+20-pcsw/2, C.Y+eph+be-fh2, 2, z);
 
 	    if (mygame->MyColor==BLACK)
 	      addTarget(new DropSource(pk[si]|BLACK,Width()+40*si,C.Y+be,40,eph));
+
+	    /*
+	    C.setColor(0xffff00);
+	    C.drawRect(40*si,C.Y+be,40,eph,false);
+	    */
 	  }
 	} // for
 
@@ -557,7 +583,7 @@ void Board::renderClock() {
       }	
   }
 
-  if (C.H > 32) {
+  if (C.H > 32 && !EditMode) {
     if (position.getAnnotation()) {
       C.consumeTop(16);
       j=16;
@@ -579,14 +605,14 @@ ChessClock * Board::getClock() {
   return(&clock);
 }
 
-int Board::getClockValue(bool white) {
-  return(clock.getValue(white?0:1));
+int Board::getClockValue2(bool white) {
+  return(clock.getValue2(white?0:1));
 }
 
-void Board::setClock(int wsecs,int bsecs,int actv,int cdown) {
+void Board::setClock2(int wmsecs,int bmsecs,int actv,int cdown) {
   global.debug("Board","setClock");
   clock_ch=1;
-  clock.setClock(wsecs,bsecs,actv,cdown);
+  clock.setClock2(wmsecs,bmsecs,actv,cdown);
   queueRepaint();
 }
 
@@ -729,12 +755,12 @@ void Board::sendMove() {
   global.debug("Board","sendMove");
   if ((sp<2)||(!mygame)) return;
   if ((allowMove)||(FreeMove)) {
-    if ((global.CheckLegality)&&(!FreeMove))
+    if (global.effectiveLegalityChecking() && !FreeMove)
       if (!position.isMoveLegalCartesian(ax[0],ay[0],ax[1],ay[1],
 					 mygame->MyColor,mygame->Variant)) {
 	char z[128];
-	sprintf(z,_("Illegal Move %c%d%c%d (Legality Checking On)"),
-		'a'+ax[0],ay[0]+1,'a'+ax[1],ay[1]+1);
+	snprintf(z,128,_("Illegal Move %c%d%c%d (Legality Checking On)"),
+		 'a'+ax[0],ay[0]+1,'a'+ax[1],ay[1]+1);
 	global.status->setText(z,15);
 	goto sendmove_cleanup;	
       }      
@@ -789,7 +815,7 @@ void Board::setCanMove(bool value) {
 
 void Board::stopClock() {
   clock_ch=1;
-  clock.setClock(clock.getValue(0),clock.getValue(1),0,1);
+  clock.setClock2(clock.getValue2(0),clock.getValue2(1),0,1);
   queueRepaint();
 }
 
@@ -852,6 +878,31 @@ void Board::walkForwardAll() {
   thaw();
 }
 
+void Board::drawJoystickCursor(GdkGC *gc) {
+  double arrow[7][2] =
+    { { 0, 0}, {0,90}, {18,68}, {40,108}, 
+      { 52,100 }, {30, 63}, {63, 63} };
+  GdkPoint myarrow[7];
+  int i;
+
+  if (jsx >= 0) {
+
+    for(i=0;i<7;i++) {
+      myarrow[i].x = (gint) (4 + jsx + arrow[i][0] * sqside / 128.0);
+      myarrow[i].y = (gint) (4 + jsy + arrow[i][1] * sqside / 128.0);
+    }
+
+    gdk_rgb_gc_set_foreground(gc,0x000000);
+    gdk_draw_polygon(yidget->window,gc,TRUE,myarrow,7);
+    for(i=0;i<7;i++) { myarrow[i].x -= 4; myarrow[i].y -= 4; }
+    gdk_rgb_gc_set_foreground(gc,0x00ff00);
+    gdk_draw_polygon(yidget->window,gc,TRUE,myarrow,7);
+    gdk_rgb_gc_set_foreground(gc,0x000000);
+    gdk_draw_polygon(yidget->window,gc,FALSE,myarrow,7);
+
+  }
+}
+
 void Board::outlineRectangle(GdkGC *gc, int x,int y,int color,int pen) {
   int i,j,k,X,Y,S;
   gdk_rgb_gc_set_foreground(gc,color);
@@ -902,8 +953,8 @@ void Board::drop(piece p) {
 				mygame->MyColor,mygame->Variant))
 	{
 	  char z[128];
-	  sprintf(z,_("Illegal Drop on %c%d (Legality Checking On)"),
-		  'a'+dropsq[0],dropsq[1]+1);
+	  snprintf(z,128,_("Illegal Drop on %c%d (Legality Checking On)"),
+		   'a'+dropsq[0],dropsq[1]+1);
 	  global.status->setText(z,15);
 	  goto drop_cleanup;
 	}
@@ -941,31 +992,31 @@ void Board::popupDropMenu(int col,int row,int sx,int sy,guint32 etime) {
   
   mi[0]=gtk_menu_item_new_with_label(_("Drop Piece:"));
   gtk_widget_set_sensitive(mi[0],FALSE);
-  mi[1]=gtk_menu_item_new();
+  mi[1]=gtk_separator_menu_item_new();
 
-  sprintf(z,_("Pawn    %d"),i=pos.getStockCount(PAWN|mygame->MyColor));
+  snprintf(z,64,_("Pawn    %d"),i=pos.getStockCount(PAWN|mygame->MyColor));
   mi[2]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[2],FALSE);
 
-  sprintf(z,_("Rook    %d"),i=pos.getStockCount(ROOK|mygame->MyColor));
+  snprintf(z,64,_("Rook    %d"),i=pos.getStockCount(ROOK|mygame->MyColor));
   mi[3]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[3],FALSE);
 
-  sprintf(z,_("Knight  %d"),i=pos.getStockCount(KNIGHT|mygame->MyColor));
+  snprintf(z,64,_("Knight  %d"),i=pos.getStockCount(KNIGHT|mygame->MyColor));
   mi[4]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[4],FALSE);
 
-  sprintf(z,_("Bishop  %d"),i=pos.getStockCount(BISHOP|mygame->MyColor));
+  snprintf(z,64,_("Bishop  %d"),i=pos.getStockCount(BISHOP|mygame->MyColor));
   mi[5]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[5],FALSE);
 
-  sprintf(z,_("Queen   %d"),i=pos.getStockCount(QUEEN|mygame->MyColor));
+  snprintf(z,64,_("Queen   %d"),i=pos.getStockCount(QUEEN|mygame->MyColor));
   mi[6]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[6],FALSE);
 
   for(i=0;i<7;i++) {
     gshow(mi[i]);
-    gtk_menu_append(GTK_MENU(popmenu),mi[i]);
+    gtk_menu_shell_append(GTK_MENU_SHELL(popmenu),mi[i]);
   }
 
   gtk_signal_connect(GTK_OBJECT(mi[2]),"activate",
@@ -988,7 +1039,7 @@ void Board::popupDropMenu(int col,int row,int sx,int sy,guint32 etime) {
 }
 
 void Board::popupProtocolMenu(int x,int y, guint32 etime) {
-  GtkWidget *mi, *hs, *popmenu;
+  GtkWidget *mi, *popmenu;
   vector<string *> *v;
   char z[64];
   unsigned int i;
@@ -1004,27 +1055,25 @@ void Board::popupProtocolMenu(int x,int y, guint32 etime) {
   v=global.protocol->getPlayerActions();
 
   for(i=0;i<v->size();i++) {
-    sprintf(z,"%s: %s",mygame->PlayerName[0], ((*v)[i])->c_str() );
+    snprintf(z,64,"%s: %s",mygame->PlayerName[0], ((*v)[i])->c_str() );
     mi=gtk_menu_item_new_with_label(z);
     gshow(mi);
-    gtk_menu_append(GTK_MENU(popmenu),mi);
+    gtk_menu_shell_append(GTK_MENU_SHELL(popmenu),mi);
 
     gtk_signal_connect(GTK_OBJECT(mi),"activate",
 		       GTK_SIGNAL_FUNC(menu_whitep),
 		       (gpointer) ( (*v)[i] ) );    
   }
 
-  mi=gtk_menu_item_new();
-  hs=gtk_hseparator_new();
-  gtk_container_add(GTK_CONTAINER(mi),hs);
-  Gtk::show(hs,mi,0);
-  gtk_menu_append(GTK_MENU(popmenu),mi);
+  mi=gtk_separator_menu_item_new();
+  gshow(mi);
+  gtk_menu_shell_append(GTK_MENU_SHELL(popmenu),mi);
 
   for(i=0;i<v->size();i++) {
-    sprintf(z,"%s: %s",mygame->PlayerName[1], ((*v)[i])->c_str() );
+    snprintf(z,64,"%s: %s",mygame->PlayerName[1], ((*v)[i])->c_str() );
     mi=gtk_menu_item_new_with_label(z);
     gshow(mi);
-    gtk_menu_append(GTK_MENU(popmenu),mi);
+    gtk_menu_shell_append(GTK_MENU_SHELL(popmenu),mi);
 
     gtk_signal_connect(GTK_OBJECT(mi),"activate",
 		       GTK_SIGNAL_FUNC(menu_blackp),
@@ -1032,19 +1081,17 @@ void Board::popupProtocolMenu(int x,int y, guint32 etime) {
   }
 
   if (global.protocol->getGameActions() != 0) {
-    mi=gtk_menu_item_new();
-    hs=gtk_hseparator_new();
-    gtk_container_add(GTK_CONTAINER(mi),hs);
-    Gtk::show(hs,mi,0);
-    gtk_menu_append(GTK_MENU(popmenu),mi);
+    mi=gtk_separator_menu_item_new();
+    gshow(mi);
+    gtk_menu_shell_append(GTK_MENU_SHELL(popmenu),mi);
 
     v=global.protocol->getGameActions();
 
     for(i=0;i<v->size();i++) {
-      sprintf(z,_("Game #%d: %s"),mygame->GameNumber, ((*v)[i])->c_str() );
+      snprintf(z,64,_("Game #%d: %s"),mygame->GameNumber, ((*v)[i])->c_str() );
       mi=gtk_menu_item_new_with_label(z);
       gshow(mi);
-      gtk_menu_append(GTK_MENU(popmenu),mi);
+      gtk_menu_shell_append(GTK_MENU_SHELL(popmenu),mi);
       
       gtk_signal_connect(GTK_OBJECT(mi),"activate",
 			 GTK_SIGNAL_FUNC(menu_gamep),
@@ -1052,15 +1099,144 @@ void Board::popupProtocolMenu(int x,int y, guint32 etime) {
     }
   }
 
+#if MAEMO
+  gtk_menu_popup(GTK_MENU(popmenu),0,0,0,0,1,etime);
+#else   
   gtk_menu_popup(GTK_MENU(popmenu),0,0,0,0,3,etime);
+#endif
+}
+
+gboolean board_joycursor(gpointer data) {
+  Board *me;
+  static double accel[5] = { 0.2, 0.4, 0.8, 1.2, 1.5 };
+  static int acc = 0;
+  static Timestamp prev;
+  Timestamp cur;
+  double elapsed;
+
+  me = (Board *) data;
+  cur = Timestamp::now();
+
+  if (me->jsx < 0) {
+    me->jsx = me->sqw / 2;
+    me->jsy = me->sqh / 2;
+    acc = 0;
+  } else {
+    elapsed = cur-prev;
+    if (elapsed < 0.50 && me->jsnt==0) {
+      me->jsx += (int) (accel[acc] * global.JSSpeed * 5.0 * (me->jsvx / 32768.0) * (elapsed/0.060));
+      me->jsy += (int) (accel[acc] * global.JSSpeed * 5.0 * (me->jsvy / 32768.0) * (elapsed/0.060));
+      if (me->jsx < 0)       me->jsx = 0;
+      if (me->jsx > me->sqw) me->jsx = me->sqw;
+      if (me->jsy < 0)       me->jsy = 0;
+      if (me->jsy > me->sqh) me->jsy = me->sqh;
+      ++acc;
+      if (acc>4) acc=4;
+      me->queueRepaint();
+    }
+  }
+
+  if (me->jsnt!=0) acc=0; 
+  me->jsnt = 0;
+  prev = cur;
+  if (global.JSMode!=1) {
+    me->jstid = -1;
+    return FALSE;
+  } else return TRUE;
+}
+
+void Board::joystickCursor(int axis, int value) {
+
+  // continuous cursor
+  if (global.JSMode == 1) {
+    if (axis==0) jsvx = value;
+    if (axis==1) jsvy = value;
+    if (jstid<0 && (jsvx!=0 || jsvy!=0)) {
+      jsnt = 1;
+      jstid = gtk_timeout_add(60,board_joycursor,(gpointer) this);
+    } else if (jstid>=0 && (jsvx==0 && jsvy==0)) {
+      gtk_timeout_remove(jstid);
+      jstid = -1;
+    }
+  }
+
+  if (effectiveFlip()) value = -value;   
+  if (value < 0) value = -1;
+  if (value > 0) value = 1;
+  
+
+  if (jx < 0) {
+    jx = 3; jy = 3; queueRepaint();
+  } else {
+
+    switch(axis) {
+    case 0: // X
+      if (jpxv != value) {
+	if (value < 0) jx--; if (jx<0) jx=0;
+	if (value > 0) jx++; if (jx>7) jx=7;
+	jpxv = value;
+      }
+      break;
+    case 1: // Y
+      if (jpyv != value) {
+	if (value < 0) jy++; if (jy>7) jy=7;
+	if (value > 0) jy--; if (jy<0) jy=0;
+	jpyv = value;
+      }
+      break;
+    }
+    queueRepaint();
+  }
+
+}
+
+void Board::joystickSelect() {
+
+  int x,y;
+
+  if (global.JSMode == 0) {
+    x = jx;
+    y = jy;
+    
+    if (sp > 1) sp=0;
+    if ( (sp == 1) && (x==ax[0]) && (y==ay[0]) )
+      sp=0;
+    else {
+      
+      if ((sp == 1) &&
+	  ( (position.getPiece(ax[0],ay[0])&COLOR_MASK) ==
+	    (position.getPiece(x,y)&COLOR_MASK)) &&
+	  (position.getPiece(x,y)!=EMPTY) && (allowMove) )
+	sp=0;
+      
+      ax[sp]=x; ay[sp]=y;
+      ++sp;
+    }
+    
+    if ((sp==2)&&(mygame)) {
+      sendMove();
+    } else {
+      premoveq=0;
+      hilite_ch=1;
+      repaint();
+    }
+  } else {
+    GdkEventButton be;
+    be.x = jsx;
+    be.y = jsy;
+    be.button = 1;
+    be.state = GDK_MOD5_MASK;
+    board_button_press_event(widget,&be,this);
+    board_button_release_event(widget,&be,this);
+  }
 }
 
 void Board::dump() {
   cerr.setf(ios::hex,ios::basefield);
   cerr.setf(ios::showbase);
 
-  cerr << "[board " << ((unsigned int)this) << "] ";
-  cerr << "game=[" << ((unsigned int)mygame) << "] ";
+  cerr << "[board " << ((uint64_t) this) << "] ";
+  cerr << "game=[" << ((uint64_t) mygame) << "] ";
 
   cerr.setf(ios::dec,ios::basefield);
   cerr << "paneid=" << getPageId() << endl;
@@ -1122,6 +1298,9 @@ void Board::drawCoordinates(GdkPixmap *dest,GdkGC *gc,int side) {
   bool ef;
   int i;
   char z[2];
+  PangoLayout *pl;
+  PangoFontDescription *pfd;
+
   z[1]=0;
   ef=effectiveFlip();
 
@@ -1141,16 +1320,25 @@ void Board::drawCoordinates(GdkPixmap *dest,GdkGC *gc,int side) {
 
   gdk_rgb_gc_set_foreground(gc,0xffffff);
 
+  pl = gtk_widget_create_pango_layout(widget, NULL);
+  pfd = pango_font_description_from_string(global.InfoFont);
+  pango_layout_set_font_description(pl, pfd);
+
   for(i=0;i<8;i++) {
     z[0]=ef?'h'-i:'a'+i;
-    gdk_draw_string(dest,f3,gc, borx+side*i+side/2, bory-4, z);
-    gdk_draw_string(dest,f3,gc, borx+side*i+side/2, morey+8*side+2*bory-4, z);
+
+    pango_layout_set_text(pl,z,-1);
+    gdk_draw_layout(dest,gc,borx+side*i+side/2, 2,pl);
+    gdk_draw_layout(dest,gc,borx+side*i+side/2, morey+8*side+bory,pl);
     z[0]=ef?'1'+i:'8'-i;
-    gdk_draw_string(dest,f3,gc, 6, bory+morey+side*i+side/2, z);
-    gdk_draw_string(dest,f3,gc, side*8+borx+6, bory+morey+side*i+side/2, z);
+    pango_layout_set_text(pl,z,-1);
+
+    gdk_draw_layout(dest,gc, 6, bory+morey+side*i+side/2, pl);
+    gdk_draw_layout(dest,gc, side*8+borx+6, bory+morey+side*i+side/2, pl);
   }
 
-
+  pango_font_description_free(pfd);
+  g_object_unref(G_OBJECT(pl));
 }
 
 void Board::composeVecBoard(GdkGC *gc) {
@@ -1188,8 +1376,6 @@ void Board::pasteVecBoard() {
 		  0, 0,Width(),Height());
 }
 
-static GdkEventMotion LastMotionEvent;
-
 /* ******************************************************************
    board_expose_event
 
@@ -1279,7 +1465,7 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
   // draw highlighted squares
 
   if (!me->gotAnimationLoop)
-    if ((me->sp)||(!me->LastMove.empty())||(me->premoveq)) {
+    if ((me->sp)||(!me->LastMove.empty())||(me->premoveq)||(me->jx>=0)) {
     
       if (!me->EditMode)
 	if ((global.HilightLastMove)&&(!me->LastMove.empty()))
@@ -1302,6 +1488,12 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
 	  me->drawBall(gc,me->pmx[0],me->pmy[0],0xff0000,5);
 	}
       }
+
+      if (global.JSMode==0 && me->jx >= 0)
+	me->outlineRectangle(gc,me->jx,me->jy,0x00ff00,8);
+
+      if (global.JSMode==1)
+	me->drawJoystickCursor(gc);
     }
   
   /* if dragging... */
@@ -1396,10 +1588,8 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
   me->sqw = 2*me->borx + 8 * me->sqside + 1;
   me->sqh = 2*me->bory + me->morey + 8 * me->sqside + 1;
 
-
-
   if ((me->currently_rendered == me->position)&&(!me->update_queue)) {
-    return 1;
+    return TRUE;
   }
 
   if (oldsqw!= me->sqw || oldsqh != me->sqh) {
@@ -1418,8 +1608,13 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
 
   memset(me->scrap,0,me->sqw * me->sqh * 3);
 
+  if (me->UpdateClockAfterConfigure) {
+    me->updateClock();
+    me->UpdateClockAfterConfigure = false;
+  }
+
   if (global.UseVectorPieces)
-    return 1;
+    return TRUE;
 
   if (me->gotAnimationLoop)
     joker=&(me->fake);
@@ -1459,7 +1654,7 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
 
   me->currently_rendered=*(joker);
 
-  return 1;
+  return TRUE;
 }
 
 /*
@@ -1535,17 +1730,20 @@ gboolean board_motion_event(GtkWidget *widget,
   Board *me;
   GdkGC *gcpgc;
   int rw,rh;
-  bool clock_too=false;
   int ww,wh,sw,sh;
   int sq,he,xc,xr;
   int bw,bh;
 
-  memcpy(&LastMotionEvent,em,sizeof(GdkEventMotion));
+  if (em!=NULL) {
+    if (!GDK_IS_WINDOW(em->window))
+      return TRUE;
+    memcpy(&LastMotionEvent,em,sizeof(GdkEventMotion));
+  }
 
   me=(Board *)data;
 
-  if ((!me->dr_fto)||(!me->dr_active)||(!em)||(!(em->state & GDK_BUTTON1_MASK)))
-    return FALSE;
+  if ((!me->dr_fto)||(!me->dr_active)||(em==NULL)||(!(em->state & GDK_BUTTON1_MASK)))
+    return TRUE;
 
   sq=he=me->sqside;
   if ((me->cur->extruded)&&(!global.UseVectorPieces)) he=me->cur->height;
@@ -1569,10 +1767,8 @@ gboolean board_motion_event(GtkWidget *widget,
     GCPbuffer=gdk_pixmap_new(widget->window, gcpw, gcph, -1);
     gdk_draw_rectangle(GCPbuffer,gcpgc,TRUE,0,0,gcpw,gcph);
     gdk_draw_pixmap(GCPbuffer,gcpgc,me->buffer,0,0,0,0,sw,sh);
-    clock_too=true;
   } else
     if (! me->dr_step) {
-      clock_too=true;
       gdk_draw_rectangle(GCPbuffer,gcpgc,TRUE,0,0,gcpw,gcph);
       gdk_draw_pixmap(GCPbuffer,gcpgc,me->buffer,0,0,0,0,sw,sh);
     }
@@ -1585,8 +1781,8 @@ gboolean board_motion_event(GtkWidget *widget,
   if (me->dr_step) {
     rw= bw - me->dr_dirty[0];
     rh= bh - me->dr_dirty[1];
-    if (rw > sq) { rw = sq; clock_too=true; }
-    if (rh > he) { rh = he; clock_too=true; }
+    if (rw > sq) rw = sq; 
+    if (rh > he) rh = he;
  
     if ((rw>0)&&(rh>0))
       gdk_draw_pixmap(GCPbuffer,gcpgc,
@@ -1594,18 +1790,20 @@ gboolean board_motion_event(GtkWidget *widget,
 		      me->dr_dirty[0],me->dr_dirty[1],
 		      me->dr_dirty[0],me->dr_dirty[1],
 		      rw,rh);
+
+    //    gdk_rgb_gc_set_foreground(gcpgc,0xffff00);
+    gdk_draw_rectangle(GCPbuffer,gcpgc,TRUE,bw+250,0,ww-(bw+250),wh);
+    gdk_draw_rectangle(GCPbuffer,gcpgc,TRUE,0,bh,bw+250,wh-bh);
+    //    gdk_rgb_gc_set_foreground(gcpgc,0);
+
   }
 
-  if (clock_too) {
-
-    gdk_draw_pixmap(GCPbuffer,
-		    widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
-		    me->clkbar,
-		    0, 0,
-		    bw, 0,250, bh );
+  gdk_draw_pixmap(GCPbuffer,
+		  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+		  me->clkbar,
+		  0, 0,
+		  bw, 0,250, bh );
     
-  }
-
   // STEP 2: erase source square (if not dragging from zhouse stock)
 
   if (! me->dr_fromstock) {
@@ -1667,7 +1865,25 @@ gboolean board_motion_event(GtkWidget *widget,
   GdkModifierType state;
 
   if (em->is_hint){
-    gdk_window_get_pointer(em->window, &x, &y, &state);
+
+    x=(int)(em->x);
+    y=(int)(em->y);
+
+    if (em->window != NULL) {
+      if (GDK_IS_WINDOW(em->window))
+	gdk_window_get_pointer(em->window, &x, &y, &state);
+      else
+	cout << "[ case 1: em->window is NOT a GDK_WINDOW ]\n";
+    } else {
+      if (me->widget->window==NULL) {
+	cout << "[ case 3: both window pointers are NULL ]\n";
+      } else if (GDK_IS_WINDOW(me->widget->window))
+	gdk_window_get_pointer(me->widget->window, &x, &y, &state);
+      else
+	cout << "[ case 2: me->widget->window is NOT a GDK_WINDOW ]\n";
+      cout << "[ And this is where it used to print a warning ]\n"; // FIXME
+    }
+
   } else {
     x=(int)(em->x);
     y=(int)(em->y);
@@ -1699,7 +1915,7 @@ gboolean board_motion_event(GtkWidget *widget,
       if (xr < 0) { xc+=xr; xr=0; }
       if (xc > me->morey) { --xc; ++xr; }
     } else { xc=0; xr=dy; }
-    
+
     me->cur->drawPiece(me->dr_p,GCPbuffer,gcpgc,dx,dy);
   }
 
@@ -1782,19 +1998,12 @@ gboolean board_button_release_event(GtkWidget *widget,
   return 1;
 }
 
-static gboolean board_drag_start_timeout(gpointer data) {
-  bool *b;
-  b=(bool *)data;
-  *b=true;
-  return FALSE;
-}
-
 gboolean board_button_press_event(GtkWidget *widget,GdkEventButton *be,
 			    gpointer data) {
   Board *me;
   int x,y;
   me=(Board *)data;
-  if (be==0) return 0;
+  if (be==NULL) return 0;
   if ((!me->canselect)&&(be->button!=3)) return 1;
 
   if ( (be->x < me->borx) || (be->y < me->bory + me->morey ) ) return 1;
@@ -1813,19 +2022,17 @@ gboolean board_button_press_event(GtkWidget *widget,GdkEventButton *be,
     DropSource *ds;
     if (be->button == 1) {
       ds=me->hitTarget((int)(be->x),(int)(be->y));      
-      if (ds) {
+      if (ds!=NULL) {
 	if (ds->dragged) {
 	  me->dr_active=true;
-	  me->dr_fto=false;
+	  me->dr_fto=true;
 	  me->dr_fromstock=true;
 	  me->dr_step=0;
 	  me->dr_p=ds->P;
 	  me->dr_ox = me->sqside / 2;
 	  me->dr_oy = me->sqside / 2;
 	  me->dr_c=0; // non-sense
-	  me->dr_r=0;
-	  gtk_timeout_add(90,board_drag_start_timeout,
-			  (gpointer)(&(me->dr_fto)));
+	  me->dr_r=0;	  
 	} else {
 	  EditBoard *eme;
 	  eme=(EditBoard *)data;
@@ -1853,16 +2060,22 @@ gboolean board_button_press_event(GtkWidget *widget,GdkEventButton *be,
 
   if (be->button == 1) {
 
+    if (be->state != GDK_MOD5_MASK) {
+      me->jx = -1;
+      me->jy = -1;
+      me->jpxv = 0;
+      me->jpyv = 0;
+      me->jsx = -1;
+      me->jsy = -1;
+    }
+
     me->dr_active=true;
-    me->dr_fto=false;
+    me->dr_fto=true;
     me->dr_fromstock=false;
     me->dr_step=0;
     me->dr_p=me->position.getPiece(x,y);
     if (me->dr_p == EMPTY)
       me->dr_active=false;
-    else
-      gtk_timeout_add(90,board_drag_start_timeout,
-		      (gpointer) (&(me->dr_fto)));
 
     if (me->sp > 1) me->sp=0;
     if ( (me->sp == 1) && (x==me->ax[0]) && (y==me->ay[0]) )
@@ -1976,6 +2189,8 @@ gboolean board_animate(gpointer data) {
   int not_over=0;
   
   me=(Board *)data;
+  if (me->cur == NULL)
+    return FALSE;
 
   dy=me->cur->side - me->cur->height;
 
@@ -2085,7 +2300,7 @@ DropSource * TargetManager::hitTarget(int x,int y) {
   for(i=0;i<j;i++)
     if (targets[i]->hit(x,y))
       return(targets[i]);
-  return 0;
+  return NULL;
 }
 
 EditBoard::EditBoard(ChessGame *cg) : Board() {
@@ -2094,13 +2309,6 @@ EditBoard::EditBoard(ChessGame *cg) : Board() {
   setGame(cg);
 }
 
-static int popx=0, popy=0;
-
-void poprun_cb(GtkMenu *m,gint *x,gint *y,gpointer data) {
-  *x=popx;
-  *y=popy;
-}
-
 static EditBoard *eb_last=0;
 
 void eb_runengine_nobm(GtkMenuItem *item,gpointer data) {
@@ -2166,7 +2374,7 @@ void EditBoard::popRunEngine(int x,int y) {
   mi.push_back(gtk_menu_item_new_with_label("Crafty..."));
   mi.push_back(gtk_menu_item_new_with_label("Sjeng..."));
   mi.push_back(gtk_menu_item_new_with_label(_("Generic XBoard Engine...")));
-  mi.push_back(gtk_menu_item_new());
+  mi.push_back(gtk_separator_menu_item_new());
 
   for(i=0;i<4;i++)
     gtk_signal_connect(GTK_OBJECT(mi[i]),"activate",
@@ -2190,26 +2398,18 @@ void EditBoard::popRunEngine(int x,int y) {
   }
 
   if (note)
-    mi.push_back(gtk_menu_item_new());
+    mi.push_back(gtk_separator_menu_item_new());
 
   for(i=0;i<mi.size();i++) {
     gshow(mi[i]);
-    gtk_menu_append(GTK_MENU(popmenu),mi[i]);
+    gtk_menu_shell_append(GTK_MENU_SHELL(popmenu),mi[i]);
   }
-
   if (note) {
     gshow(note);
-    gtk_menu_append(GTK_MENU(popmenu), note);
+    gtk_menu_shell_append(GTK_MENU_SHELL(popmenu), note);
   }
 
-  gdk_window_get_origin(widget->window,&popx,&popy);
-
-  popx+=x;
-  popy+=y;
-
-  gtk_menu_popup(GTK_MENU(popmenu),0,0,
-		 (GtkMenuPositionFunc) poprun_cb,
-		 0,1,time(0));
+  gtk_menu_popup(GTK_MENU(popmenu),NULL,NULL,NULL,NULL,1,gtk_get_current_event_time());
 }
 
 void EditBoard::renderClock() {
@@ -2222,54 +2422,72 @@ void EditBoard::renderClock() {
   if (!mygame) return;
 
   // scratch-board pieces and buttons
+
   C.consumeTop(10);
-  i = 32;
 
-  for(k=0;k<6;k++) {
-    C.drawButton(10+i*k, C.Y, i, i, NULL, 0, 0xffffff, 0x505070);
-    C.drawButton(10+i*k, C.Y+i+8, i, i, NULL, 0, 0xffffff, 0x505070);
-    
-    global.vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y,WHITE|(k+1));
-    global.vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y+i+8,BLACK|(k+1));
-    addTarget(new DropSource(WHITE|(k+1),Width()+10+i*k,C.Y,i,i));
-    addTarget(new DropSource(BLACK|(k+1),Width()+10+i*k,C.Y+i+8,i,i));
+  if (C.H > 80+48) {
+    i = 32;
+    for(k=0;k<6;k++) {
+      C.drawButton(10+i*k, C.Y, i, i, NULL, 0, 0xffffff, 0x505070);
+      C.drawButton(10+i*k, C.Y+i+8, i, i, NULL, 0, 0xffffff, 0x505070);
+      
+      global.vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y,WHITE|(k+1));
+      global.vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y+i+8,BLACK|(k+1));
+      addTarget(new DropSource(WHITE|(k+1),Width()+10+i*k,C.Y,i,i));
+      addTarget(new DropSource(BLACK|(k+1),Width()+10+i*k,C.Y+i+8,i,i));
+    }    
+    C.consumeTop(i*2+16);
+  } else if (C.H > 24) {
+    i = 16;
+    for(k=0;k<6;k++) {
+      C.drawButton(10+i*k, C.Y, i, i, NULL, 0, 0xffffff, 0x505070);
+      C.drawButton(10+i*(6+k), C.Y, i, i, NULL, 0, 0xffffff, 0x505070);
+      
+      global.vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y,WHITE|(k+1));
+      global.vpieces.drawPiece(clkbar,gc, i, 10+i*(6+k), C.Y,BLACK|(k+1));
+      addTarget(new DropSource(WHITE|(k+1),Width()+10+i*k,C.Y,i,i));
+      addTarget(new DropSource(BLACK|(k+1),Width()+10+i*(6+k),C.Y,i,i));
+    }    
+    C.consumeTop(24);
   }
+
+  if (C.H > 28) {
+    j=C.drawButton(10,C.Y,-1,20,_("Empty"),2,0xffffff, 0x505070);
+    addTarget(new DropSource(EMPTY,Width()+10,C.Y,j,20,false));
+    
+    k=10+j+10;
+    j=C.drawButton(k,C.Y,-1,20,_("Initial Position"),2,0xffffff,0x505070);
+    
+    addTarget(new DropSource(WHITE|BLACK,Width()+k,C.Y,j,20,false));
+    
+    k+=10+j;
+    j=C.drawButton(k,C.Y,-1,20,_("From FEN"),2,0xffffff,0x505070);
+    
+    addTarget(new DropSource(WASPAWN,Width()+k,C.Y,j,20,false));
   
-  C.consumeTop(i*2+16);
-  
-  j=C.drawButton(10,C.Y,-1,20,_("Empty"),2,0xffffff, 0x505070);
-  addTarget(new DropSource(EMPTY,Width()+10,C.Y,j,20,false));
-  
-  k=10+j+10;
-  j=C.drawButton(k,C.Y,-1,20,_("Initial Position"),2,0xffffff,0x505070);
-  
-  addTarget(new DropSource(WHITE|BLACK,Width()+k,C.Y,j,20,false));
+    C.consumeTop(28);
+  }
+  if (C.H > 20) {
+    j=C.drawButton(10,C.Y,-1,20,_("Run Engine..."),
+		   2,0xffffff,0xa07050);
+    addTarget(new DropSource(WHITE,Width()+10,C.Y,j,20,false));
+    
+    k=j+20;
+    j+=20+C.stringWidth(2,_("Side to move: "));
+    
+    C.drawButton( k,C.Y,j-k+10+24,20,NULL,2,0xffffff,0x505070);
+    addTarget(new DropSource(BLACK,Width()+k,C.Y,j-k+10+24,20,false));
+    
+    C.drawString( k+10, C.Y+2, 2, _("Side to move: "));
+    
+    C.setColor( mygame->getSideHint() ? 0xffffff : 0);
+    C.drawEllipse( 10+j+3, C.Y+3, 14, 14, true);
+    C.setColor(0xffffff);
+    C.drawEllipse( 10+j+3, C.Y+3, 14, 14, false);  
+    
+    C.consumeTop(20);
+  }
 
-  k+=10+j;
-  j=C.drawButton(k,C.Y,-1,20,_("From FEN"),2,0xffffff,0x505070);
-
-  addTarget(new DropSource(WASPAWN,Width()+k,C.Y,j,20,false));
-  
-  C.consumeTop(28);
-  
-  j=C.drawButton(10,C.Y,-1,20,_("Run Engine..."),
-		 2,0xffffff,0xa07050);
-  addTarget(new DropSource(WHITE,Width()+10,C.Y,j,20,false));
-
-  k=j+20;
-  j+=20+C.stringWidth(2,_("Side to move: "));
-
-  C.drawButton( k,C.Y,j-k+10+24,20,NULL,2,0xffffff,0x505070);
-  addTarget(new DropSource(BLACK,Width()+k,C.Y,j-k+10+24,20,false));
-
-  C.drawString( k+10, C.Y+14, 2, _("Side to move: "));
-
-  C.setColor( mygame->getSideHint() ? 0xffffff : 0);
-  C.drawEllipse( 10+j+3, C.Y+3, 14, 14, true);
-  C.setColor(0xffffff);
-  C.drawEllipse( 10+j+3, C.Y+3, 14, 14, false);  
-
-  C.consumeTop(20);
 }
 
 void EditBoard::flipSide() {
@@ -2290,8 +2508,8 @@ void EditBoard::applyFEN(string &s) {
   p.setFEN(s.c_str());
 
   mygame->editEmpty();
-  mygame->updatePosition(p,1,p.sidehint ? 0 : 1,
-			 0,0,"<FEN>",false);  
+  mygame->updatePosition2(p,1,p.sidehint ? 0 : 1,
+			  0,0,"<FEN>",false);  
 }
 
 GetFENDialog::GetFENDialog(EditBoard *_owner) 
@@ -2326,7 +2544,7 @@ GetFENDialog::GetFENDialog(EditBoard *_owner)
   gtk_box_pack_start(GTK_BOX(hb),cancel,TRUE,TRUE,0);
   gtk_widget_grab_default(ok);
 
-  Gtk::show(ok,cancel,hb,hs,e,v,0);
+  Gtk::show(ok,cancel,hb,hs,e,v,NULL);
   setDismiss(GTK_OBJECT(cancel),"clicked");
 
   gtk_signal_connect(GTK_OBJECT(ok),"clicked",
