@@ -1,4 +1,4 @@
-/* $Id: board.h,v 1.50 2003/07/10 04:42:15 bergo Exp $ */
+/* $Id: board.h,v 1.61 2008/02/08 14:25:50 bergo Exp $ */
 
 /*
 
@@ -101,7 +101,7 @@ class TargetManager {
 
 class RootBoard {
  protected:
-  void clockString(int val,char *z);
+  void clockString(int val,char *z,int maxlen);
 };
 
 // I just can't work out the correct syntax to put the
@@ -148,8 +148,8 @@ class Board : public WidgetProxy,
   void        setFlipInversion(bool v);
   bool        getFlipInversion();
   ChessClock *getClock();
-  int         getClockValue(bool white);
-  void        setClock(int wsecs,int bsecs,int actv,int cdown);
+  int         getClockValue2(bool white);
+  void        setClock2(int wmsecs,int bmsecs,int actv,int cdown);
   void        setInfo(int idx,char *s);
   void        setInfo(int idx,string &s);
   void        setCanMove(bool value);
@@ -192,6 +192,9 @@ class Board : public WidgetProxy,
 
   virtual void renderClock();
 
+  void joystickCursor(int axis, int value);
+  void joystickSelect();
+
  protected:
   GdkPixmap *clkbar;
   GdkGC *clkgc;
@@ -210,7 +213,6 @@ class Board : public WidgetProxy,
   PieceSet *cur;
   GdkPixmap *buffer;
   GdkGC *wgc;
-  GdkFont *f1,*f2,*f3;
   rgbptr scrap;
 
   mblock<unsigned char> M[2];
@@ -221,6 +223,7 @@ class Board : public WidgetProxy,
   bool flipped;
   bool repaint_due;
   bool update_queue;
+  bool UpdateClockAfterConfigure;
 
   int sel_color;
   int frozen_lvl;
@@ -242,6 +245,8 @@ class Board : public WidgetProxy,
 
   // selection/highlight
   int ax[2],ay[2],sp;
+  int jx,jy,jpxv,jpyv; // discrete joystick cursor
+  int jsx,jsy,jsvx,jsvy,jstid,jsnt; // smooth joystick cursor
 
   // premove
   int premoveq;
@@ -276,6 +281,7 @@ class Board : public WidgetProxy,
   void popupProtocolMenu(int x,int y, guint32 etime);
   void sendMove();
   void outlineRectangle(GdkGC *gc, int x,int y,int color,int pen);
+  void drawJoystickCursor(GdkGC *gc);
   void drawBall(GdkGC *gc, int x,int y,int color,int radius);
   void drop(piece p);
   void drawCoordinates(GdkPixmap *dest,GdkGC *gc,int side);
@@ -286,6 +292,7 @@ class Board : public WidgetProxy,
   void shadyText(int x,int y, int f, char *z);
 
   friend gboolean board_animate(gpointer data);
+  friend gboolean board_joycursor(gpointer data);
   friend gboolean vec_board_animate(gpointer data);
 
   friend gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
@@ -303,6 +310,9 @@ class Board : public WidgetProxy,
 				     GdkEventMotion *em,
 				     gpointer data);
 
+  friend gboolean gtkDgtnixEvent(GIOChannel* channel, GIOCondition cond, gpointer data);
+  
+
   friend void drop_callbackP(GtkMenuItem *item,gpointer data);
   friend void drop_callbackR(GtkMenuItem *item,gpointer data);
   friend void drop_callbackN(GtkMenuItem *item,gpointer data);
@@ -313,6 +323,10 @@ class Board : public WidgetProxy,
   friend void menu_blackp(GtkMenuItem *item, gpointer data);
   friend void menu_gamep(GtkMenuItem *item, gpointer data);
 
+#if MAEMO
+  friend void tap_and_hold_cb(GtkWidget *widget, gpointer user_data);
+#endif
+
 };
 
 class EditBoard : public Board {
@@ -342,5 +356,27 @@ class GetFENDialog : public ModalDialog {
   friend void getfen_ok(GtkWidget *w, gpointer data);
 };
 
+
+/* friend declarations for GCC 4.x */
+gboolean board_animate(gpointer data);
+gboolean vec_board_animate(gpointer data);
+gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,gpointer data);
+gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,gpointer data);
+gboolean board_button_press_event(GtkWidget *widget,GdkEventButton *be,gpointer data);
+gboolean board_button_release_event(GtkWidget *widget,GdkEventButton *be,gpointer data);
+gboolean board_motion_event(GtkWidget *widget,GdkEventMotion *em,gpointer data);
+void drop_callbackP(GtkMenuItem *item,gpointer data);
+void drop_callbackR(GtkMenuItem *item,gpointer data);
+void drop_callbackN(GtkMenuItem *item,gpointer data);
+void drop_callbackB(GtkMenuItem *item,gpointer data);
+void drop_callbackQ(GtkMenuItem *item,gpointer data);
+void menu_whitep(GtkMenuItem *item, gpointer data);
+void menu_blackp(GtkMenuItem *item, gpointer data);
+void menu_gamep(GtkMenuItem *item, gpointer data);
+void eb_runengine_bm(GtkMenuItem *item,gpointer data);
+void eb_runengine_nobm(GtkMenuItem *item,gpointer data);
+void getfen_ok(GtkWidget *w, gpointer data);
+gboolean board_joycursor(gpointer data);
+
 #endif
 
