@@ -1,10 +1,10 @@
-/* $Id: bugpane.cc,v 1.7 2002/05/24 22:05:47 bergo Exp $ */
+/* $Id: bugpane.cc,v 1.18 2008/02/08 14:25:50 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -23,10 +23,11 @@
 
 */
 
-#include <iostream.h>
+#include <iostream>
 #include <string.h>
 #include "bugpane.h"
 #include "global.h"
+#include "eboard.h"
 
 string BugPane::BugTell;
 
@@ -34,13 +35,8 @@ string BugPane::BugTell;
 
 BareBoard::BareBoard() : WidgetProxy() {
   global.addPieceClient(this);
-  myfont=global.loadFont(EF_InfoFont);
-  if (!myfont) {
-    cerr << "<BareBoard::BareBoard> can't load font.\n";
-    exit(2);
-  }
-  Names[0]="None";
-  Names[1]="None";
+  Names[0]=_("None");
+  Names[1]=_("None");
   
   widget=gtk_drawing_area_new();
   gtk_widget_set_events(widget,GDK_EXPOSURE_MASK);
@@ -61,7 +57,6 @@ BareBoard::BareBoard() : WidgetProxy() {
 
 BareBoard::~BareBoard() {
   global.removePieceClient(this);
-  gdk_font_unref(myfont);
   if (pixbuf)
     gdk_pixmap_unref(pixbuf);
 }
@@ -99,8 +94,8 @@ void BareBoard::updateClock() {
   repaint();
 }
 
-void BareBoard::setClock(int wsecs,int bsecs,int actv,int cdown) {
-  clock.setClock(wsecs,bsecs,actv,cdown);
+void BareBoard::setClock2(int wmsecs,int bmsecs,int actv,int cdown) {
+  clock.setClock2(wmsecs,bmsecs,actv,cdown);
   update();
 }
 
@@ -147,13 +142,8 @@ gboolean bareboard_expose(GtkWidget *widget,GdkEventExpose *ee,
 
   me=(BareBoard *)data;
 
-  if (! me->pixbuf)
-    me->pixbuf=gdk_pixmap_new(widget->window,SCRAP_WIDTH,SCRAP_WIDTH,-1);
-
   gdk_window_get_size(widget->window,&ww,&wh);
   zh=wh;
-  if (ww>SCRAP_WIDTH) ww=SCRAP_WIDTH;
-  if (wh>SCRAP_WIDTH) wh=SCRAP_WIDTH;
   sqside=ww/8;
 
   if ( (!global.UseVectorPieces) && (me->pset->extruded) )
@@ -161,10 +151,25 @@ gboolean bareboard_expose(GtkWidget *widget,GdkEventExpose *ee,
 
   if (wh<ww) sqside=wh/8;
 
+  if (me->pixbuf) {
+    if (ww != me->pixw || wh != me->pixh) {
+      gdk_pixmap_unref(me->pixbuf);
+      me->pixbuf=0;
+      me->pixw=ww;
+      me->pixh=wh;
+    }
+  }
+
+  if (! me->pixbuf)
+    me->pixbuf=gdk_pixmap_new(widget->window,ww,wh,-1);
+
   gc=gdk_gc_new(me->pixbuf);
   gdk_rgb_gc_set_foreground(gc,0);
   gdk_draw_rectangle(me->pixbuf,gc,TRUE,0,0,ww,wh);
 
+  me->C.prepare(widget,me->pixbuf,gc,0,0,ww,wh);
+  me->C.setFont(0,global.InfoFont);
+
   if (sqside > 8) {
 
     if (global.UseVectorPieces) {
@@ -206,32 +211,35 @@ gboolean bareboard_expose(GtkWidget *widget,GdkEventExpose *ee,
     
     x=sqside*8+10;
 
-    gdk_rgb_gc_set_foreground(gc,0xffffff);
-    gdk_draw_string(me->pixbuf,me->myfont,gc,x,20,"Bughouse: Partner Game");
-    gdk_draw_string(me->pixbuf,me->myfont,gc,x+1,20,"Bughouse: Partner Game");
+    int fh = me->C.fontHeight(0);
+
+    me->C.setColor(0xffffff);
+    me->C.drawString(x,10,0,_("Bughouse: Partner Game"));
+    me->C.drawString(x+1,10,0,_("Bughouse: Partner Game"));
 
     // white's clock
 
     if (me->clock.getActive() == -1) {
-      gdk_draw_rectangle(me->pixbuf,gc,TRUE,x-5,(flip?40:zh-5)-16, 150, 20);
-      gdk_rgb_gc_set_foreground(gc,0);
+      gdk_draw_rectangle(me->pixbuf,gc,TRUE,x-5,(flip?40:zh-5-fh), 150, 20);
+      me->C.setColor(0);
     }
     
-    me->clockString(me->clock.getValue(0),t);
-    sprintf(z,"White: %s - %s",me->Names[0].c_str(),t);
-    gdk_draw_string(me->pixbuf,me->myfont,gc,x,flip?40:zh-5,z);
+    me->clockString(me->clock.getValue2(0),t,64);
+    snprintf(z,256,_("White: %s - %s"),me->Names[0].c_str(),t);
+
+    me->C.drawString(x,flip?40:zh-5-fh,0,z);
 
     // black's clock
 
-    gdk_rgb_gc_set_foreground(gc,0xffffff);
+    me->C.setColor(0xffffff);
     if (me->clock.getActive() == 1) {
-      gdk_draw_rectangle(me->pixbuf,gc,TRUE,x-5,(flip?zh-5:40)-16, 150, 20);
-      gdk_rgb_gc_set_foreground(gc,0);
+      gdk_draw_rectangle(me->pixbuf,gc,TRUE,x-5,(flip?zh-5-fh:40), 150, 20);
+      me->C.setColor(0);
     }
 
-    me->clockString(me->clock.getValue(1),t);
-    sprintf(z,"Black: %s - %s",me->Names[1].c_str(),t);
-    gdk_draw_string(me->pixbuf,me->myfont,gc,x,flip?zh-5:40,z);
+    me->clockString(me->clock.getValue2(1),t,64);
+    snprintf(z,256,_("Black: %s - %s"),me->Names[1].c_str(),t);
+    me->C.drawString(x,flip?zh-5-fh:40,0,z);
 
     // paint stock
 
@@ -262,16 +270,16 @@ gboolean bareboard_expose(GtkWidget *widget,GdkEventExpose *ee,
     // paint ptells
 
     x=sqside*8+10;
-    gdk_rgb_gc_set_foreground(gc,global.Colors.TextBright);
-    gdk_draw_string(me->pixbuf,me->myfont,gc,x,60+sqside,"Partner Tells:");
+    me->C.setColor(global.Colors.TextBright);
+    me->C.drawString(x,40+sqside,0,_("Partner Tells:"));
   
     if (!me->PTells.empty()) {
-      gdk_rgb_gc_set_foreground(gc,global.Colors.PrivateTell);
+      me->C.setColor(global.Colors.PrivateTell);
       i=zh-20-5-sqside;
 
       pti=me->PTells.end();
-      for(pti--;i>(79+sqside);i-=20, pti--) {
-	gdk_draw_string(me->pixbuf,me->myfont,gc, x,i, (*pti).c_str() );
+      for(pti--;i>(59+sqside);i-=20, pti--) {
+	me->C.drawString(x,i,0,(*pti).c_str() );
 	if (pti == me->PTells.begin())
 	  break;
       }
@@ -314,7 +322,7 @@ BugPane::BugPane() {
   gray[2].red=gray[2].green=gray[2].blue=0xcccc;
   gray[3].red=gray[3].green=gray[3].blue=0x8888;
   gray[4].red=gray[4].green=gray[4].blue=0xaaaa;
-  style=gtk_style_new();
+  style=gtk_style_copy( gtk_widget_get_default_style() );
   for(i=0;i<5;i++)
     style->bg[i]=gray[i];
 
@@ -328,29 +336,28 @@ BugPane::BugPane() {
     c=i/6;
     r=i%6;
     gtk_table_attach_defaults(GTK_TABLE(tbl),qb[i],c,c+1,r,r+1);
-    gtk_widget_show(qb[i]);
+    gshow(qb[i]);
     gtk_signal_connect(GTK_OBJECT(qb[i]),"clicked",
 		       GTK_SIGNAL_FUNC(bug_ptell),(gpointer)(stuff[i]));
   }
 
   gtk_box_pack_start(GTK_BOX(v), tbl, FALSE, FALSE, 0);
 
-  gtk_widget_show(tbl);
-  gtk_widget_show(v);
+  Gtk::show(tbl,v,NULL);
 }
 
 void BugPane::stopClock() {
-  board->setClock(0,0,0,1);
+  board->setClock2(0,0,0,1);
 }
 
 void BugPane::reset() {
   Position p;
 
   freeze();
-  board->setClock(0,0,0,1);
+  board->setClock2(0,0,0,1);
   board->setPosition(p);
-  board->setWhite("None");
-  board->setBlack("None");
+  board->setWhite(_("None"));
+  board->setBlack(_("None"));
   thaw();
   
 }
@@ -371,8 +378,8 @@ void BugPane::setPosition(Position &pos) {
   board->update();
 }
 
-void BugPane::setClock(int wsecs,int bsecs,int actv,int cdown) {
-  board->setClock(wsecs,bsecs,actv,cdown);
+void BugPane::setClock2(int wmsecs,int bmsecs,int actv,int cdown) {
+  board->setClock2(wmsecs,bmsecs,actv,cdown);
 }
 
 void BugPane::freeze() { board->freeze(); }
@@ -381,7 +388,7 @@ void BugPane::thaw()   { board->thaw(); }
 void bug_ptell(GtkWidget *b,gpointer data) {
   char z[256],x[256],c;
   
-  strcpy(z,(char *)data);
+  g_strlcpy(z,(char *)data,256);
 
   if ((z[0]=='-')||(z[0]=='+')) {
     BugPane::BugTell=z;
@@ -390,7 +397,7 @@ void bug_ptell(GtkWidget *b,gpointer data) {
     if (BugPane::BugTell.empty())
       BugPane::BugTell=z;
     else {
-      strcpy(x,BugPane::BugTell.c_str());
+      g_strlcpy(x,BugPane::BugTell.c_str(),256);
       c=x[strlen(x)-1];
       if ((c=='+')||(c=='-'))
 	BugPane::BugTell+=z;
@@ -399,8 +406,8 @@ void bug_ptell(GtkWidget *b,gpointer data) {
     }
   }
 
-  if (global.network) {
-    sprintf(z,"ptell %s",BugPane::BugTell.c_str());
-    global.network->writeLine(z);
+  if (global.protocol) {
+    snprintf(z,256,"ptell %s",BugPane::BugTell.c_str());
+    global.protocol->sendUserInput(z);
   }
 }
