@@ -1,10 +1,10 @@
-/* $Id: chess.cc,v 1.79 2003/02/18 02:49:48 bergo Exp $ */
+/* $Id: chess.cc,v 1.93 2008/02/08 14:25:50 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -59,8 +59,6 @@ ChessGame::ChessGame() {
     protodata[i]=0;
 
   GameNumber=-1;
-  STime=0;
-  Inc=0;
   Rated=0;
   Variant=REGULAR;
   memset(PlayerName[0],0,64);
@@ -91,13 +89,12 @@ ChessGame::ChessGame(int _number,int _tyme,int _inc, int _rated,
   int i;
 
   GameNumber=_number;
-  STime=_tyme;
-  Inc=_inc;
+  timecontrol.setIcs(_tyme,_inc);
   Rated=_rated;
   Variant=_variant;
   PlayerName[0][63]=PlayerName[1][63]=0;
-  strncpy(PlayerName[0],p1,64);
-  strncpy(PlayerName[1],p2,64);
+  g_strlcpy(PlayerName[0],p1,64);
+  g_strlcpy(PlayerName[1],p2,64);
   last_half_move=-1;
   myboard=0;
   cursor=moves.end();
@@ -125,8 +122,6 @@ ChessGame::ChessGame(ChessGame *src) {
   bool cset=false;
 
   GameNumber = src->GameNumber;
-  STime = src->STime;
-  Inc = src->Inc;
   Rated = src->Rated;
   Variant = src->Variant;
   strcpy(PlayerName[0],src->PlayerName[0]);
@@ -139,6 +134,7 @@ ChessGame::ChessGame(ChessGame *src) {
   StopClock=0;
   Rating[0][0]=Rating[1][0]=0;
   LocalEdit=false;
+  timecontrol = src->timecontrol;
 
   Loaded=true;
   PGNSource[0]=0;
@@ -176,36 +172,41 @@ void ChessGame::setFree() {
 }
 
 void ChessGame::acknowledgeInfo() {
-  char s[64];
+  char s[64],tz[128];
 
-  if (!myboard)
+  if (myboard==NULL)
     return;
   myboard->freeze();
 
-  if ((!STime)&&(GameNumber > 7000)) {
-    strcpy(s,info0);
-  } else
-    if ((STime)||(Inc)) {
-      sprintf(s,_("Game #%d - %d %d %s"),
-	      GameNumber,
-	      STime/60,Inc,
-	      Rated?_("rated"):_("unrated"));
-    } else {
-      sprintf(s,_("Game #%d - untimed %s"),
-	      GameNumber,
-	      Rated?_("rated"):_("unrated"));
-    }
-
+  // P2P and engine games
+  if (GameNumber > 7000 && GameNumber < 9000) {
+    timecontrol.toString(s,64);
+  } else if (GameNumber >= 9000) { // client-side games - rated/unrated unknown
+    timecontrol.toShortString(tz,128);
+    snprintf(s,64,_("Game #%d - %s"), // TRANSLATE
+	    GameNumber,tz);
+  } else { // live ICS games
+    timecontrol.toShortString(tz,128);
+    snprintf(s,64,_("Game #%d - %s - %s"), // TRANSLATE?
+	    GameNumber,tz,
+	    Rated?_("rated"):
+	          _("unrated"));
+  }
   
   myboard->setInfo(0,s);
 
   strcpy(s,variantName(Variant));
   myboard->setInfo(1,s);
 
+  if (!moves.empty())
+    myboard->setInfo(4,moves.back().getMaterialString(Variant));
+
   if (over)
     showResult();
 
+  myboard->invalidate();
   myboard->thaw();
+  myboard->updateClock();
 }
 
 void ChessGame::showResult() {
@@ -219,7 +220,7 @@ void ChessGame::showResult() {
   default: strcpy(str,"(*) "); break;
   }
   if (strlen(ereason)>2)
-    strcat(str,ereason);
+    g_strlcat(str,ereason,256);
   myboard->setInfo(3,str);
 }
 
@@ -261,17 +262,17 @@ void ChessGame::fixExamineZigZag(Position &suspect) {
 }
 
 // updates clocks and just-below-clocks strings
-void ChessGame::updateClockAndInfo(int wclock, int bclock,
-				   int blacktomove, char *infoline,
-				   bool sndflag)
+void ChessGame::updateClockAndInfo2(int wclockms, int bclockms,
+				    int blacktomove, char *infoline,
+				    bool sndflag)
 {
   if (myboard) {
     myboard->freeze();
 
     if (StopClock)
-      myboard->setClock(wclock,bclock,blacktomove?2:-2,clock_regressive);
+      myboard->setClock2(wclockms,bclockms,blacktomove?2:-2,clock_regressive);
     else
-      myboard->setClock(wclock,bclock,blacktomove?1:-1,clock_regressive);
+      myboard->setClock2(wclockms,bclockms,blacktomove?1:-1,clock_regressive);
     
     if (infoline)
       myboard->setInfo(2,infoline);
@@ -286,8 +287,8 @@ void ChessGame::updateClockAndInfo(int wclock, int bclock,
   }
 }
 
-void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
-			       int wclock,int bclock,char *infoline, 
+void ChessGame::updatePosition2(Position &p,int movenum,int blacktomove,
+			       int wclockms,int bclockms,char *infoline, 
 			       bool sndflag) {
   global.debug("ChessGame","updatePosition");
 
@@ -300,9 +301,9 @@ void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
     global.bugpane->setPosition(p);
 
     if (StopClock)
-      global.bugpane->setClock(wclock,bclock,blacktomove?2:-2,clock_regressive);
+      global.bugpane->setClock2(wclockms,bclockms,blacktomove?2:-2,clock_regressive);
     else
-      global.bugpane->setClock(wclock,bclock,blacktomove?1:-1,clock_regressive);
+      global.bugpane->setClock2(wclockms,bclockms,blacktomove?1:-1,clock_regressive);
 
     global.bugpane->thaw();
   }
@@ -310,7 +311,7 @@ void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
   if (last_half_move>=0)
     if (! moves.empty() )
       if (p == moves.back() ) {
-	updateClockAndInfo(wclock,bclock,blacktomove,infoline,
+	updateClockAndInfo2(wclockms,bclockms,blacktomove,infoline,
 			   sndflag);
 	return;
       }
@@ -327,8 +328,8 @@ void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
   cursor--;
   last_half_move=(movenum*2)+(blacktomove?0:1);
 
-  updateClockAndInfo(wclock,bclock,blacktomove,infoline,
-		     sndflag);
+  updateClockAndInfo2(wclockms,bclockms,blacktomove,infoline,
+		      sndflag);
 
   if (mymovelist)
     mymovelist->updateList(moves);
@@ -338,12 +339,12 @@ void ChessGame::incrementActiveClock(int secs) {
   ChessClock *c;
   if (myboard) {
     c = myboard->getClock();
-    c->incrementClock(0,secs);
+    c->incrementClock2(0,secs*1000);
   }
 }
 
 void ChessGame::fireWhiteClock(int wval,int bval) {
-  myboard->setClock(wval,bval,-1,clock_regressive);
+  myboard->setClock2(wval*1000,bval*1000,-1,clock_regressive);
 }
 
 void ChessGame::setBoard(Board *b) {
@@ -590,8 +591,7 @@ void ChessGame::endGame(char *reason,GameResult _result) {
     myboard->stopClock();
 
   over=1;
-  strncpy(ereason,reason,128);
-  ereason[127]=0;
+  g_strlcpy(ereason,reason,128);
 
   showResult();
   if (myboard)
@@ -623,7 +623,7 @@ void ChessGame::updateGame(list<Position> &gamedata) {
   }
 
   // should fix the adding of FEN fields on followed games
-  if (moves.empty() && Variant!=WILD) {
+  if (moves.empty() && IS_NOT_WILD(Variant)) {
     moves.push_back(Position());
     hmn=0;
   }
@@ -639,12 +639,11 @@ void ChessGame::dump() {
   cerr.setf(ios::hex,ios::basefield);
   cerr.setf(ios::showbase);
 
-  cerr << "[game " << ((unsigned int)this) << "] ";
+  cerr << "[game " << ((uint64_t) this) << "] ";
 
   cerr.setf(ios::dec,ios::basefield);
 
   cerr << "game#=" << GameNumber << " ";
-  cerr << "time=(" << STime << "," << Inc << ") ";
   cerr << "rated=" << Rated << " ";
   cerr << "white=" << PlayerName[0] << " ";
   cerr << "black=" << PlayerName[1] << " ";
@@ -653,14 +652,14 @@ void ChessGame::dump() {
   cerr.setf(ios::hex,ios::basefield);
   cerr.setf(ios::showbase);
 
-  cerr << "board=[" << ((unsigned int)myboard) << "]" << endl;
+  cerr << "board=[" << ((uint64_t)myboard) << "]" << endl;
 }
 
 char * ChessGame::getPlayerString(int index) {
   index%=2;
   PrivateString[0]=0;
   if ((global.ShowRating)&&(strlen(Rating[index])))
-    sprintf(PrivateString,"%s (%s)",PlayerName[index],Rating[index]);
+    snprintf(PrivateString,96,"%s (%s)",PlayerName[index],Rating[index]);
   else
     strcpy(PrivateString,PlayerName[index]);
   return(PrivateString);
@@ -669,33 +668,55 @@ char * ChessGame::getPlayerString(int index) {
 // ---- PGN
 
 void ChessGame::guessInfoFromPGN() {
-  static char *tcsep = "+ -/\t";
   const char *cp;
 
-  if ((cp=pgn.get("White")))
-    strcpy(PlayerName[0],cp);
+  cp=pgn.get("White");
+  if (cp!=NULL) g_strlcpy(PlayerName[0],cp,64);
 
-  if ((cp=pgn.get("Black")))
-    strcpy(PlayerName[1],cp);
+  cp=pgn.get("Black");
+  if (cp!=NULL) g_strlcpy(PlayerName[1],cp,64);
 
   cp=pgn.get("Result");
-  if (cp) {
-    if (cp[0]=='0') { result=BLACK_WIN; return; }
-    if (cp[1]=='/') { result=DRAW; return; }
-    if (cp[0]=='1') { result=WHITE_WIN; return; }
-    result=UNDEF;
+  result = UNDEF;
+  if (cp!=NULL) {
+    if (cp[0]=='0')      result=BLACK_WIN;
+    else if (cp[1]=='/') result=DRAW;
+    else if (cp[0]=='1') result=WHITE_WIN;
   }
 
-  if ((cp=pgn.get("TimeControl"))) {
-    tstring t;
-    t.set((char *) cp);
-    STime = t.tokenvalue(tcsep);
-    Inc   = t.tokenvalue(tcsep);
+  timecontrol.mode = TC_NONE;
+  cp=pgn.get("TimeControl");
+  if (cp!=NULL) {
+    int a,b;
+    ExtPatternMatcher FischerClock, XMoves, SecsPerMove;
+    
+    FischerClock.set("%N+%N*");
+    XMoves.set("%N/%N*");
+    SecsPerMove.set("0+%N*");
+    
+    if (SecsPerMove.match(cp)) {
+
+      a = atoi(SecsPerMove.getNToken(0));
+      timecontrol.setSecondsPerMove(a);
+
+    } else if (FischerClock.match(cp)) {
+
+      a = atoi(FischerClock.getNToken(0));
+      b = atoi(FischerClock.getNToken(1));
+      timecontrol.setIcs(a,b);
+
+    } else if (XMoves.match(cp)) {
+
+      a = atoi(FischerClock.getNToken(0));
+      b = atoi(FischerClock.getNToken(1));
+      timecontrol.setXMoves(a,b);
+
+    }
   }
 
-  if ((cp=pgn.get("Variant"))) {
-    Variant = variantFromName((char *)cp);
-  }
+  cp=pgn.get("Variant");
+  if (cp!=NULL) Variant = variantFromName(cp);
+
 }
 
 void ChessGame::guessPGNFromInfo() {
@@ -717,9 +738,9 @@ void ChessGame::guessPGNFromInfo() {
     x[0] = toupper(x[0]);
 
     // ICS Rated Chess Match
-    sprintf(y,"ICS %s %s Match",
-	    Rated?"Rated":"Unrated",
-	    x);
+    snprintf(y,256,"ICS %s %s Match",
+	     Rated?"Rated":"Unrated",
+	     x);
     pgn.set("Event",y);
     break;
   case GS_Engine:
@@ -727,7 +748,7 @@ void ChessGame::guessPGNFromInfo() {
     x[0] = toupper(x[0]);
 
     // Engine Chess Match
-    sprintf(y,"Engine %s Match",x);
+    snprintf(y,256,"Engine %s Match",x);
     pgn.set("Event",y);
     break;
   case GS_Other:
@@ -739,8 +760,8 @@ void ChessGame::guessPGNFromInfo() {
 
   pgn.set("Site","?");
   
-  sprintf(z,"%d.%.2d.%.2d",1900+snow->tm_year,
-	  1+snow->tm_mon,snow->tm_mday);
+  snprintf(z,64,"%d.%.2d.%.2d",1900+snow->tm_year,
+	   1+snow->tm_mon,snow->tm_mday);
   
   pgn.set("Date",z);
   pgn.set("Round","?");
@@ -763,10 +784,8 @@ void ChessGame::guessPGNFromInfo() {
       rz[strlen(rz)]=Rating[1][i];
   if (strlen(rz)) pgn.set("BlackElo",rz);
 
-  if ( (STime > 0) || (Inc > 0) ) {
-    sprintf(rz,"%d+%d",STime,Inc);
-    pgn.set("TimeControl",rz);
-  }
+  timecontrol.toPGN(rz,64);
+  pgn.set("TimeControl",rz);
 
   if (Variant != REGULAR) {
     pgn.set("Variant",(char *) variantName(Variant));
@@ -812,7 +831,7 @@ bool ChessGame::savePGN(char *filename, bool append) {
   ofstream f(path.c_str(),append? ios::app : ios::out);
 
   if (!f) {
-    sprintf(z,_("savePGN failed: %s"),filename);
+    snprintf(z,512,_("savePGN failed: %s"),filename);
     global.status->setText(z,10);
     return false;
   }
@@ -866,8 +885,9 @@ bool ChessGame::savePGN(char *filename, bool append) {
 
   f.close();
 
-  sprintf(z,_("--- %s game to PGN file %s"),
-	  append?_("Appended"):_("Wrote"),filename);
+  snprintf(z,512,_("--- %s game to PGN file %s"),
+	   append?_("Appended"):
+	          _("Wrote"),filename);
   global.output->append(z,0xc0ff00);
   return true;
 }
@@ -925,10 +945,10 @@ bool ChessGame::ParsePgnGame(zifstream &f,
   Legacy0.set("%%eboard:variant:%s*");
   Legacy1.set("%%eboard:clock/r:%n:%n:%n*");  
   Legacy2.set("%%eboard:clue:*");
-  Binder.add(&Legacy0,&Legacy1,&Legacy2,0);
+  Binder.add(&Legacy0,&Legacy1,&Legacy2,NULL);
 
   Result.set("*%N-%N*");
-  Binder2.add(&Result,0);
+  Binder2.add(&Result,NULL);
 
   ChessGame::initGlyphs();
 
@@ -942,7 +962,7 @@ bool ChessGame::ParsePgnGame(zifstream &f,
 
     if (indexonly) {
       game->Loaded=false;
-      strcpy(game->PGNSource,filename);
+      g_strlcpy(game->PGNSource,filename,256);
       game->SourceOffset = pstart;
     }
   }
@@ -977,15 +997,17 @@ bool ChessGame::ParsePgnGame(zifstream &f,
 
       if (Legacy1.match()) {
 	if (game) {
-	  game->STime=atoi(Legacy1.getNToken(0));
-	  game->Inc=atoi(Legacy1.getNToken(1));
+	  int a,b;
+	  a = atoi(Legacy1.getNToken(0));
+	  b = atoi(Legacy1.getNToken(1));
+	  game->timecontrol.setIcs(a,b);
 	  game->Rated=atoi(Legacy1.getNToken(2));
 	}
 	continue;
       } // match Legacy1
 
       if (Legacy2.match()) {
-	strncpy(omega,Legacy2.getStarToken(0),128);
+	g_strlcpy(omega,Legacy2.getStarToken(0),128);
 	if (omega[strlen(omega)-1]=='\n')
 	  omega[strlen(omega)-1]=0;
 	continue;
@@ -1138,8 +1160,8 @@ bool ChessGame::ParsePgnGame(zifstream &f,
 	    g.clearAnnotation();
 
 	    g.moveAnyNotation(alpha,color,game ? game->Variant : v);
-	    sprintf(beta,"%d%s %s",1+halfmove/2,
-		    (color==WHITE)?".":". ...",alpha);
+	    snprintf(beta,256,"%d%s %s",1+halfmove/2,
+		     (color==WHITE)?".":". ...",alpha);
 	    g.setLastMove(beta);
 	    gl.push_back(g);
 	  }
@@ -1229,20 +1251,20 @@ bool ChessGame::loadMoves() {
   zifstream f(PGNSource);
 
   if (!f) {
-    sprintf(z,_("can't load PGN move text from %s (error opening file)"),PGNSource);
+    snprintf(z,512,_("can't load PGN move text from %s (error opening file)"),PGNSource);
     global.output->append(z,0xc0ff00);
     return false;
   }
 
   if (!f.seekg(SourceOffset)) {
-    sprintf(z,_("can't seek to offset %lu of %s"),SourceOffset,PGNSource);
+    snprintf(z,512,_("can't seek to offset %lu of %s"),SourceOffset,PGNSource);
     global.output->append(z,0xc0ff00);
     f.close();
     return false;
   }
   
   if (!ParsePgnGame(f,PGNSource,false,-1,Variant,this)) {
-    strcpy(z,_("error parsing PGN data"));
+    g_strlcpy(z,_("error parsing PGN data"),512);
     global.output->append(z,0xc0ff00);
     f.close();
     return false;
@@ -1298,7 +1320,8 @@ void ChessGame::LoadPGN(char *filename) {
 	if ( ii == g_ids.end() ) id_space_empty = true;
       }
 
-    } // parse success
+    } else
+      break;
 
     if (! (count%5) ) pw->setProgress(count);
   }
@@ -1312,34 +1335,32 @@ GameResult ChessGame::getResult() {
   return(over?result:UNDEF);
 }
 
-variant ChessGame::variantFromName(char *p) {
+variant ChessGame::variantFromName(const char *p) {
   variant v=REGULAR;
-  if (!strcmp(p,variantName(CRAZYHOUSE))) v=CRAZYHOUSE;
-  else
-    if (!strcmp(p,variantName(BUGHOUSE)))   v=BUGHOUSE;
-    else
-      if (!strcmp(p,variantName(WILD)))       v=WILD;
-      else
-	if (!strcmp(p,variantName(SUICIDE)))    v=SUICIDE;
-	else
-	  if (!strcmp(p,variantName(LOSERS)))     v=LOSERS;
-	  else
-	    if (!strcmp(p,variantName(GIVEAWAY)))   v=GIVEAWAY;
-	    else
-	      if (!strcmp(p,variantName(ATOMIC)))     v=ATOMIC;
+  variant allv[10] = {CRAZYHOUSE, BUGHOUSE, WILD,   SUICIDE,    LOSERS,
+		      GIVEAWAY,   ATOMIC,   WILDFR, WILDCASTLE, WILDNOCASTLE};
+  int i;
+
+  for(i=0;i<10;i++)
+    if (!strcmp(p,variantName(allv[i])))
+      v = allv[i];
+
   return v;
 }
 
 const char * ChessGame::variantName(variant v) {
   switch(v) {
-  case REGULAR:    return("chess");
-  case CRAZYHOUSE: return("crazyhouse");
-  case SUICIDE:    return("suicide");
-  case BUGHOUSE:   return("bughouse");
-  case WILD:       return("wild");
-  case LOSERS:     return("losers");
-  case GIVEAWAY:   return("giveaway");
-  case ATOMIC:     return("atomic");
+  case REGULAR:      return("chess");
+  case CRAZYHOUSE:   return("crazyhouse");
+  case SUICIDE:      return("suicide");
+  case BUGHOUSE:     return("bughouse");
+  case WILD:         return("wild");
+  case LOSERS:       return("losers");
+  case GIVEAWAY:     return("giveaway");
+  case ATOMIC:       return("atomic");
+  case WILDFR:       return("fischerandom");
+  case WILDCASTLE:   return("wildcastle");
+  case WILDNOCASTLE: return("nocastle");
   default: return("unknown");
   }
 }
@@ -1449,14 +1470,15 @@ PGNEditInfoDialog::PGNEditInfoDialog(ChessGame *src) :
 				 GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
 
   clist=gtk_clist_new(2);
+  gtk_widget_set_style(clist, gtk_widget_get_default_style() );
+  gtk_clist_set_column_min_width(GTK_CLIST(clist),0,64);
+  gtk_clist_set_column_min_width(GTK_CLIST(clist),1,64);
   gtk_clist_set_shadow_type(GTK_CLIST(clist),GTK_SHADOW_IN);
   gtk_clist_set_selection_mode(GTK_CLIST(clist),GTK_SELECTION_SINGLE);
   gtk_clist_set_column_title(GTK_CLIST(clist),0,_("Key"));
   gtk_clist_set_column_title(GTK_CLIST(clist),1,_("Value"));
   gtk_clist_column_titles_passive(GTK_CLIST(clist));
   gtk_clist_column_titles_show(GTK_CLIST(clist));
-  gtk_clist_set_column_min_width(GTK_CLIST(clist),0,64);
-  gtk_clist_set_column_min_width(GTK_CLIST(clist),1,64);
 
   gtk_box_pack_start(GTK_BOX(v),sw,TRUE,TRUE,0);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
@@ -1498,7 +1520,7 @@ PGNEditInfoDialog::PGNEditInfoDialog(ChessGame *src) :
   gtk_widget_grab_default(closeb);
 
   Gtk::show(closeb,bb,hs,l[0],l[1],en[0],en[1],setb,hb2,
-	    hb,del,sw,clist,v,0);
+	    hb,del,sw,clist,v,NULL);
   setDismiss(GTK_OBJECT(closeb),"clicked");
   
   populate();
@@ -1540,8 +1562,8 @@ void pgnedit_set(GtkWidget *w, gpointer data) {
   PGNEditInfoDialog *me;
   me=(PGNEditInfoDialog *)data;
   char a[64],b[64];
-  strncpy(a,gtk_entry_get_text(GTK_ENTRY(me->en[0])),64);
-  strncpy(b,gtk_entry_get_text(GTK_ENTRY(me->en[1])),64);
+  g_strlcpy(a,gtk_entry_get_text(GTK_ENTRY(me->en[0])),64);
+  g_strlcpy(b,gtk_entry_get_text(GTK_ENTRY(me->en[1])),64);
   if (strlen(a) && strlen(b)) {
     me->obj->pgn.set((const char *)a,b);
     me->populate();
