@@ -43,11 +43,9 @@
 #define USE_IPPROTO_TCP 1
 #define HAVE_GTK_GTK_H 1
 #define HAVE_GDK_GDKKEYSYMS_H 1
-#define HAVE_DGTNIX_H 1
-#define WITH_DGT_BOARD 1
-#define DATADIR "/usr/share"
-#define EBOARD_PREFIX "/usr"
-#define VERSION "1.0.3"
+#define DATADIR "/usr/local/share"
+#define EBOARD_PREFIX "/usr/local"
+#define VERSION "1.1.1"
 #define ENABLE_NLS 1
 
 #endif
