@@ -1,10 +1,10 @@
-/* $Id: dlg_connect.h,v 1.4 2003/02/05 19:57:21 bergo Exp $ */
+/* $Id: dlg_connect.h,v 1.5 2007/01/01 18:29:03 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -49,6 +49,13 @@ class ConnectDialog : public WidgetProxy {
 			     gpointer data);
 };
 
+void dlg_connect_ok(GtkWidget *w,gpointer data);
+void dlg_connect_cancel(GtkWidget *w,gpointer data);
+
+void dlgconn_rowsel(GtkCList *clist,int row,int column,GdkEventButton *eb,
+		    gpointer data);
+
+
 class EditEngineBookmarksDialog : public ModalDialog,
 				  public UpdateInterface
 {
@@ -80,6 +87,16 @@ class EditEngineBookmarksDialog : public ModalDialog,
 
 };
 
+void eebmd_rowsel(GtkCList *clist,int row,int column,GdkEventButton *eb,
+		  gpointer data);
+void eebmd_rowunsel(GtkCList *clist,int row,int column,GdkEventButton *eb,
+		    gpointer data);
+
+void eebmd_rm1(GtkWidget *w, gpointer data);
+void eebmd_rmall(GtkWidget *w, gpointer data);
+void eebmd_apply(GtkWidget *w, gpointer data);
+void eebmd_edittc(GtkWidget *w, gpointer data);
+
 typedef EditEngineBookmarksDialog * EEBMDp;
 
 #endif
