@@ -1,10 +1,10 @@
-/* $Id: dlg_prefs.h,v 1.24 2003/01/20 03:30:20 bergo Exp $ */
+/* $Id: dlg_prefs.h,v 1.32 2008/02/22 07:32:15 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2008 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -31,48 +31,68 @@
 #include "widgetproxy.h"
 
 class PreferencesDialog : public ModalDialog,
-                          public SoundEventChangeListener
+                          public SoundEventChangeListener,
+                          public JoystickListener
 {
  public:
   PreferencesDialog();
-  ~PreferencesDialog();
+  virtual ~PreferencesDialog();
 
   void SoundEventChanged();
 
+  virtual void joystickEvent(JoystickEventType jet, int number, int value);
+
  private:
   GtkWidget *tabposb[4],*efont[NFONTS],*fontdlg,*plainb,
-    *showratb, *autologb, *sbacke, /* *allob[2], */ 
+    *showratb, *autologb, *showts, *sbacke, *special[4], /* *allob[2], */ 
     *asp, *aso, *afn, *sgb, *hsb, *chsb, *coct, *dhsb, *aqbar, *lowtime,
     *wget, *smoothb, *xfont[NFONTS], 
     *sndon[N_SOUND_EVENTS],*sndtest[N_SOUND_EVENTS],*sndedit[N_SOUND_EVENTS],
-    *sndd[N_SOUND_EVENTS];
+    *sndd[N_SOUND_EVENTS], *jcl, *jctl, *jmode, *jspeed;
   BoxedLabel *fm[NFONTS];
   SoundEvent sndcopy[N_SOUND_EVENTS];
   ColorButton *lsq, *dsq, *textcb[11];
   TextPreview *preview;
   int FontBeingEdited;
+  int jsval[5], jstate;
 
   static char *FontSample[NFONTS];
 
-  void ResolveFont(char *src,char *dest);
-
   void Apply();
   void ApplyCheckBox(GtkWidget *cb,int *curval,int *ch1, int *ch2);
   void ApplyEntry(GtkWidget *entry,char *curval,int sz, int *ch1, int *ch2);
   void ApplyColorButton(ColorButton *cb,int *curval,int *ch1, int *ch2);
 
+  void formatJoystickDescription();
+
   friend void prefs_ok(GtkWidget *w,gpointer data);
   friend void prefs_apply(GtkWidget *w,gpointer data);
 
   friend void prefs_frevert(GtkWidget *w,gpointer data);
   friend void prefs_cfont(GtkWidget *w,gpointer data);
   friend void prefs_fok(GtkWidget *w,gpointer data);
+  friend void prefs_fcancel(GtkWidget *w,gpointer data);
 
   friend void prefs_sndtest(GtkWidget *w,gpointer data);
   friend void prefs_sndedit(GtkWidget *w,gpointer data);
 
   friend void prefs_defcolor(GtkWidget *w,gpointer data);
+
+  friend void prefs_joyctl(GtkWidget *w,gpointer data);
 };
 
+void prefs_ok(GtkWidget *w,gpointer data);
+void prefs_apply(GtkWidget *w,gpointer data);
+
+void prefs_frevert(GtkWidget *w,gpointer data);
+void prefs_cfont(GtkWidget *w,gpointer data);
+void prefs_fok(GtkWidget *w,gpointer data);
+void prefs_fcancel(GtkWidget *w,gpointer data);
+
+void prefs_sndtest(GtkWidget *w,gpointer data);
+void prefs_sndedit(GtkWidget *w,gpointer data);
+
+void prefs_defcolor(GtkWidget *w,gpointer data);
+void prefs_joyctl(GtkWidget *w,gpointer data);
 
 #endif
