@@ -1,10 +1,10 @@
-/* $Id: global.cc,v 1.80 2003/07/03 19:47:53 bergo Exp $ */
+/* $Id: global.cc,v 1.98 2008/02/22 07:32:16 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2008 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -35,15 +35,13 @@
 #include <sys/wait.h>
 #include <time.h>
 #include <signal.h>
+#include <dlfcn.h>
 
 #define GLOBAL_CC 1
 
 #include "global.h"
 #include "text.h"
-
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "chess.h"
 #include "tstring.h"
@@ -53,6 +51,11 @@
 #include "pieces.h"
 #include "eboard.h"
 
+
+/* Global variables needed to cleanly close the driver */
+extern int (*dgtnixClose)();
+extern bool DgtInit;
+extern void *dgtnix_dll_handle;
 Global global;
 
 // stream ops
@@ -118,7 +121,7 @@ Global::Global() {
   chandler=0;
   promotion=0;
   ebook=0;
-  skgraph=0;
+  skgraph2=0;
   inputhistory=0;
   bmlistener=0;
   qbcontainer=0;
@@ -162,8 +165,10 @@ Global::Global() {
   PlainSquares=0;
   LightSqColor=0xe7cf93;
   DarkSqColor=0x9e8661;
-
+  
+  ShowTimestamp = 0;
   ShowRating=1;
+  SpecialChars = 3;
 
   UseVectorPieces=0;
   CheckLegality=0;
@@ -192,6 +197,18 @@ Global::Global() {
   IcsAllObPlayed   = 0;
   IcsAllObObserved = 0;
 
+  JoystickFD = -1;
+
+  JSCursorAxis = 0;
+  JSBrowseAxis = 2;
+  JSMoveButton = 0;
+  JSNextTabButton = 5;
+  JSPrevTabButton = 4;  
+  JSMode  = 1;
+  JSSpeed = 4;
+
+  joycapture = NULL;
+
   ShowQuickbar=1;
   LowTimeWarningLimit=5;
   RetrieveChannelNames=1;
@@ -271,6 +288,15 @@ Global::Global() {
   RCKeys.push_back("IcsAllObPlayed");  // 44
   RCKeys.push_back("IcsAllObObserved");
   RCKeys.push_back("P2PName");         // 46
+  RCKeys.push_back("ShowTimestamp");
+  RCKeys.push_back("SpecialChars");// 48
+  RCKeys.push_back("JSCAxis");
+  RCKeys.push_back("JSBAxis"); // 50
+  RCKeys.push_back("JSMButton");
+  RCKeys.push_back("JSNTButton"); // 52
+  RCKeys.push_back("JSPTButton");
+  RCKeys.push_back("JSMode"); // 54
+  RCKeys.push_back("JSSpeed");
 
   PopupHelp = false;
 }
@@ -324,6 +350,15 @@ void Global::removeBoard(Board *b) {
   //  cerr << "<Global::removeBoard> ** board not found\n";
 }
 
+bool Global::effectiveLegalityChecking() {
+  if (CheckLegality) return true;
+
+  if (protocol != NULL)
+    return(protocol->requiresLegalityChecking());
+
+  return false;
+}
+
 void Global::statOS() {
   FILE *p;
   p=popen("uname -s","r");
@@ -346,8 +381,11 @@ void Global::ensureDirectories() {
   if (env.Home.empty()) {
     cerr << _("[eboard] ** no $HOME") << endl;
     return;
+  } else if (strlen(env.Home.c_str()) > 230) {
+    cerr << _("[eboard] ** $HOME is too long") << endl;
+    return;
   }
-  sprintf(z,"%s/.eboard",env.Home.c_str());
+  snprintf(z,256,"%s/.eboard",env.Home.c_str());
 
   tdir=opendir(z);
   if (tdir==NULL)
@@ -356,11 +394,11 @@ void Global::ensureDirectories() {
     closedir(tdir);
     
   if (createDir(z)) return;
-  sprintf(z,"%s/.eboard/craftylog",env.Home.c_str());
+  snprintf(z,256,"%s/.eboard/craftylog",env.Home.c_str());
   createDir(z);
-  sprintf(z,"%s/.eboard/eng-out",env.Home.c_str());
+  snprintf(z,256,"%s/.eboard/eng-out",env.Home.c_str());
   createDir(z);
-  sprintf(z,"%s/.eboard/scripts",env.Home.c_str());
+  snprintf(z,256,"%s/.eboard/scripts",env.Home.c_str());
   createDir(z);
 }
 
@@ -418,7 +456,7 @@ void Global::readRC() {
     if (!p) continue;
     if (p->at(0)=='#') continue;
 
-    for(j=0;j<=46;j++) {
+    for(j=0;j<=55;j++) {
       if (! p->compare(RCKeys[j]) ) {
 	switch(j) {
 	case  0: HilightLastMove =t.tokenvalue(sep); break;
@@ -490,9 +528,18 @@ void Global::readRC() {
 	case 45: IcsAllObObserved    =t.tokenvalue(sep); break;
 	case 46: p=t.token(sep2); memset(P2PName,0,64);
                  p->copy(P2PName,63); break;
+	case 47: ShowTimestamp       =t.tokenvalue(sep); break;
+	case 48: SpecialChars        =t.tokenvalue(sep); break;
+	case 49: JSCursorAxis = t.tokenvalue(sep); break;
+	case 50: JSBrowseAxis = t.tokenvalue(sep); break;
+	case 51: JSMoveButton = t.tokenvalue(sep); break;
+	case 52: JSNextTabButton = t.tokenvalue(sep); break;
+	case 53: JSPrevTabButton = t.tokenvalue(sep); break;
+	case 54: JSMode  = t.tokenvalue(sep); break;
+	case 55: JSSpeed = t.tokenvalue(sep); break;
 	} // switch
       } // compare
-    } // for j 0..41
+    } // for j 0..55
   } // while getline
 
   rc.close();
@@ -584,6 +631,15 @@ void Global::writeRC() {
   rc << RCKeys[44] << div << IcsAllObPlayed << endl;
   rc << RCKeys[45] << div << IcsAllObObserved << endl;
   rc << RCKeys[46] << div << P2PName << endl;
+  rc << RCKeys[47] << div << ShowTimestamp << endl;
+  rc << RCKeys[48] << div << SpecialChars << endl;
+  rc << RCKeys[49] << div << JSCursorAxis << endl;
+  rc << RCKeys[50] << div << JSBrowseAxis << endl;
+  rc << RCKeys[51] << div << JSMoveButton << endl;
+  rc << RCKeys[52] << div << JSNextTabButton << endl;
+  rc << RCKeys[53] << div << JSPrevTabButton << endl;
+  rc << RCKeys[54] << div << JSMode << endl;
+  rc << RCKeys[55] << div << JSSpeed << endl;
 
   rc.close();
 }
@@ -593,7 +649,7 @@ ChessGame * Global::getGame(int num) {
   for(gi=GameList.begin();gi!=GameList.end();gi++)
     if ( (*(*gi)) == num )
       return(*gi);
-  return 0;
+  return NULL;
 }
 
 int Global::nextFreeGameId(int base) {
@@ -617,6 +673,12 @@ void Global::WrappedMainQuit() {
   }
   QuitPending=0;
   signal(SIGCHLD,SIG_DFL); // prevent the crash reported by gcp
+  /* close dgtnix driver and dll */
+  if(DgtInit)
+    {
+      dgtnixClose();
+      dlclose(dgtnix_dll_handle);
+    }
   gtk_main_quit();
 }
 
@@ -733,7 +795,7 @@ void Global::debug(char *klass,char *method,char *data) {
   if (env.Home.empty())
     return;
 
-  sprintf(z,"%s/DEBUG.eboard",env.Home.c_str());
+  snprintf(z,256,"%s/DEBUG.eboard",env.Home.c_str());
 
   ofstream f(z,ios::app);
   if (!f) return;
@@ -764,7 +826,7 @@ void Global::LogAppend(char *msg) {
     msg=_("(message obfuscated -- password mode ?)");
 
   if (CommLog) {
-    sprintf(z,"%s/LOG.eboard",env.Home.c_str());
+    snprintf(z,256,"%s/LOG.eboard",env.Home.c_str());
 
     ofstream f(z,ios::app);
     if (!f) return;
@@ -860,6 +922,62 @@ Notebook * Global::getNotebook() {
   return(ebook);
 }
 
+char * Global::filter(char *s) {
+  int i,j;
+  string t;
+  gunichar uc;
+  char *c;
+
+  if (SpecialChars==0) return s; // no filtering
+
+  j = strlen(s);
+  for(i=0;i<j;i++)
+    if (s[i] & 0x80 != 0)
+      break;
+  if (i==j) return(s); // ascii-clean, just return it
+  
+  for(c=s;*c!=0;c=g_utf8_next_char(c)) {
+    uc = g_utf8_get_char(c);
+    if (uc<128)
+      t.append( 1, (char) uc );
+    else {
+      switch(SpecialChars) {
+      case 1: break; // truncate
+      case 2: t.append(1,'_'); break; // underscores
+      case 3: unicodeNormalize(t,uc); break; // canonical decomposition
+      }
+    }
+  }
+  
+  return(strdup(t.c_str()));
+}
+
+void Global::unicodeNormalize(string &dest, gunichar src) {
+  gunichar *tmp;
+  gsize i,len;
+  tmp = g_unicode_canonical_decomposition(src, &len);
+  for(i=0;i<len;i++) {
+    if (tmp[i] > 128) {
+      switch(tmp[i]) {
+      case 0x300:             tmp[i] = '`';  break; // grave
+      case 0xb4:  case 0x301: tmp[i] = '\''; break; // acute
+      case 0x302:             tmp[i] = '^';  break; // circumflex
+      case 0x303:             tmp[i] = '~';  break; // tilde
+      case 0xb8:  case 0x327: tmp[i] = ',';  break; // cedil
+      case 0x2d9: case 0x307: tmp[i] = '.';  break; // dot above
+      case 0x308:             tmp[i] = '\"'; break; // diaeresis
+      case 0x323:             tmp[i] = '.';  break; // dot below
+      default: 
+	//cout << "not found: " << ((int) tmp[i]) << endl; 
+	tmp[i] = '_'; 
+      }
+    }
+    dest.append( 1, (char) (tmp[i]&0x7f) );
+  }
+
+  g_free(tmp);  
+}
+
 void Global::gatherConsoleState() {
   list<DetachedConsole *>::iterator i;
 
@@ -875,39 +993,6 @@ void * Global::safeMalloc(int nbytes) {
   return(malloc(nbytes + (nbytes % 4)));
 }
 
-GdkFont * Global::loadFont(EboardFont i) {
-  GdkFont *f;
-
-  switch(i) {
-
-  case EF_PlayerFont: // player font (default 18 pt)
-    f=gdk_font_load(PlayerFont);
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--17-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--14-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("helvetica");
-    break;       
-
-  case EF_ClockFont: // clock font (default 34 pt)
-    f=gdk_font_load(ClockFont);
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--34-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--25-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--24-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("helvetica");
-    break;
-
-  case EF_InfoFont: // info font (default 12 pt)
-    f=gdk_font_load(InfoFont);
-    if (!f) f=gdk_font_load("-*-helvetica-medium-r-normal--11-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("-*-helvetica-medium-r-normal--10-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("helvetica");
-    break;
-  default:
-    f=NULL;
-  }
-  return f;
-
-}
-
 void Global::setPieceSet(string &filename,bool chgPieces,bool chgSquares) {
   string oldp,olds;
   PieceSet *oldset=0;
@@ -1026,7 +1111,7 @@ void ChannelSplitter::getChannels(char *ipaddr) {
   if (! global.RetrieveChannelNames)
     return;
 
-  sprintf(destname,"/tmp/eboard-chlist-%s-%d.tmp", ipaddr, getuid() );
+  snprintf(destname,512,"/tmp/eboard-chlist-%s-%d.tmp", ipaddr, getuid() );
   chlist=destname;
 
   if (stat(destname, &age)==0) {
@@ -1037,7 +1122,7 @@ void ChannelSplitter::getChannels(char *ipaddr) {
       goto cs_gc_use_current;
   }
 
-  sprintf(url,"http://eboard.sourceforge.net/ics/%s.txt",ipaddr);
+  snprintf(url,512,"http://eboard.sourceforge.net/ics/%s.txt",ipaddr);
 
   kid=fork();
   if (kid==0) {
@@ -1092,11 +1177,11 @@ const char * ChannelSplitter::getChannelTitle(int n) {
   j=channels.size();
   for(i=0;i<j;i++) {
     if (n==channels[i].number) {
-      sprintf(z,"#%d: %s",n,channels[i].name.c_str());
+      snprintf(z,128,"#%d: %s",n,channels[i].name.c_str());
       return z;
     }
   }
-  sprintf(z,"#%d",n);
+  snprintf(z,128,"#%d",n);
   return z;
 }
 
@@ -1117,7 +1202,7 @@ void ChannelSplitter::createPane(int ch) {
   if (!nb) return;  
   op=new Text();
   
-  sprintf(z,"%s",getChannelTitle(ch) );
+  snprintf(z,64,"%s",getChannelTitle(ch) );
 
   op->show();
   nb->addPage(op->widget,z,-200-ch,true);
@@ -1184,6 +1269,13 @@ void ChannelSplitter::updateChannelScrollBacks() {
     panes[i]->updateScrollBack();
 }
 
+void ChannelSplitter::updateFont() {
+  int i,j;
+  j=panes.size();
+  for(i=0;i<j;i++)
+    panes[i]->updateFont();
+}
+
 TerminalColor::TerminalColor() {
   TextDefault   = 0xeeeeee;
   TextBright    = 0xffffff;
@@ -1224,6 +1316,10 @@ WindowGeometry::WindowGeometry() {
   setNull();
 }
 
+void WindowGeometry::print() {
+  cout << "X,Y,W,H = " << X << "," << Y << "," << W << "," << H << endl;
+}
+
 void WindowGeometry::retrieve(GtkWidget *w) {
   gint a[7];  
   gdk_window_get_geometry(w->window,a,a+1,a+2,a+3,a+4);
@@ -1252,6 +1348,10 @@ void WindowGeometry::read(tstring &t) {
 
 // --------
 
+Desktop::Desktop() {
+  clear();
+}
+
 void Desktop::clear() {
   vector<WindowGeometry *>::iterator i;
   vector<string *>::iterator j;
@@ -1269,7 +1369,7 @@ void Desktop::clear() {
 
   consoles.clear();
   cfilters.clear();
-
+  PanePosition = 0;
 }
 
 void Desktop::read(tstring &t) {
@@ -1278,7 +1378,7 @@ void Desktop::read(tstring &t) {
   wGames.read(t);
   wLocal.read(t);
   wAds.read(t);
-  global.Desk.PanePosition = t.tokenvalue(sep); /* BUG */
+  global.Desk.PanePosition = t.tokenvalue(sep);
 }
 
 void Desktop::writeConsoles(ostream &s, const char *key) {
@@ -1325,7 +1425,7 @@ void Desktop::spawnConsoles(TextSet *ts) {
     dc->show();
     dc->restorePosition(consoles[i]);
     if (cfilters[i]->size()) {
-      strcpy(tmp,cfilters[i]->c_str());
+      g_strlcpy(tmp,cfilters[i]->c_str(),512);
       dc->setFilter(tmp);
     }
   }
