@@ -1,10 +1,10 @@
-/* $Id: global.h,v 1.81 2007/01/20 23:28:57 bergo Exp $ */
+/* $Id: global.h,v 1.87 2008/02/22 07:32:17 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2008 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -130,7 +130,6 @@ class ChannelSplitter : public SigChildHandler
 
   void parseChannelList();
   const char * getChannelTitle(int n);
-
   vector<Text *>  panes;
   vector<int>     numbers;
   string chlist;
@@ -269,6 +268,8 @@ class Global : public ChannelSplitter
   void addPieceClient(PieceChangeListener *pcl);
   void removePieceClient(PieceChangeListener *pcl);
 
+  char * filter(char *s);
+
   // free with free() as usual
   static void * safeMalloc(int nbytes);
 
@@ -293,7 +294,7 @@ class Global : public ChannelSplitter
   ConnectionHandler  *chandler;
   PieceProvider      *promotion;
   Notebook           *ebook;
-  SeekGraph          *skgraph;
+  SeekGraph2         *skgraph2;
   History            *inputhistory;
   BookmarkListener   *bmlistener;
   UpdateInterface    *qbcontainer;
@@ -331,6 +332,7 @@ class Global : public ChannelSplitter
   int LightSqColor;
   int DarkSqColor;
 
+  int ShowTimestamp;
   int ShowRating;
   int ScrollBack;
   int FicsAutoLogin;
@@ -340,6 +342,7 @@ class Global : public ChannelSplitter
   int EnableSounds;
   int UseVectorPieces;
   int CheckLegality;
+  int SpecialChars; // 0=none 1=truncate 2=underscore 3=soft translate (a^)
 
   int SplitChannels;
   int ChannelsToConsoleToo;
@@ -361,9 +364,6 @@ class Global : public ChannelSplitter
   int IcsAllObPlayed;
   int IcsAllObObserved;
 
-  int SeekTableSortCol;
-  int SeekTableSortDir;
-
   int JSCursorAxis;
   int JSBrowseAxis;
   int JSMoveButton;
@@ -413,6 +413,8 @@ class Global : public ChannelSplitter
 
   void playOther(int i);
 
+  void unicodeNormalize(string &dest, gunichar src);
+
   int MainLevel;
   int QuitPending;
 
