@@ -1,10 +1,10 @@
-/* $Id: mainwindow.cc,v 1.118 2003/11/22 04:55:29 bergo Exp $ */
+/* $Id: mainwindow.cc,v 1.138 2008/02/22 14:35:43 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2008 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -38,6 +38,10 @@
 
 #include "config.h"
 
+#ifdef HAVE_LINUX_JOYSTICK_H
+#include <linux/joystick.h>
+#endif
+
 #include "icon-eboard.xpm"
 
 #include "back1.xpm"
@@ -150,6 +154,7 @@ MainWindow::MainWindow() {
   consolecopy=0;
   scriptlist=0;
   ims=0;
+  jpd = 0;
 
   io_tag = -1;
 
@@ -161,9 +166,9 @@ MainWindow::MainWindow() {
 
   widget=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_widget_set_events(widget,GDK_KEY_PRESS_MASK);
-  gtk_window_set_default_size(GTK_WINDOW(widget),800,600);
   setTitle("eboard");
   gtk_widget_realize(widget);
+  gtk_window_resize(GTK_WINDOW(widget),800,600);
   RefWindow=widget->window;
 
   tooltips=GTK_TOOLTIPS(gtk_tooltips_new());
@@ -194,6 +199,12 @@ MainWindow::MainWindow() {
   /* promotion menu */
   promote=new PromotionPicker(widget->window);
 
+  /* restore main window geometry */
+  if (!global.Desk.wMain.isNull()) {
+    gtk_window_move(GTK_WINDOW(widget),global.Desk.wMain.X,global.Desk.wMain.Y);
+    gtk_window_resize(GTK_WINDOW(widget),global.Desk.wMain.W,global.Desk.wMain.H);
+  }
+
   whb=gtk_hbox_new(FALSE,0);
   gtk_box_pack_start (GTK_BOX (v), whb, FALSE, TRUE, 0);
 
@@ -218,14 +229,15 @@ MainWindow::MainWindow() {
   }
   notebook->show();
 
-  gtk_paned_pack1 (GTK_PANED (P), notebook->widget, TRUE, FALSE);
-
   SN=gtk_notebook_new();
+  gtk_paned_pack1 (GTK_PANED (P), notebook->widget, TRUE, TRUE);
   gtk_paned_pack2 (GTK_PANED (P), SN, TRUE, TRUE);
 
-  gtk_paned_set_position(GTK_PANED(P),7000);
-  gtk_paned_set_gutter_size(GTK_PANED(P),8);
-  gtk_paned_set_handle_size(GTK_PANED(P),8);
+  /* restore pane divider position */
+  if (global.Desk.PanePosition)
+    gtk_paned_set_position(GTK_PANED(P),global.Desk.PanePosition);    
+  else
+    gtk_paned_set_position(GTK_PANED(P),7000);
 
   global.lowernotebook = SN;
 
@@ -242,7 +254,7 @@ MainWindow::MainWindow() {
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK(SN), GTK_POS_RIGHT);
 
   xconsole->show();
-  Gtk::show(tl1,SN,P,0);
+  Gtk::show(tl1,SN,P,NULL);
 
   icsout->addTarget(xconsole);
 
@@ -288,7 +300,7 @@ MainWindow::MainWindow() {
   gtk_widget_set_events(inputbox,(GdkEventMask)(gtk_widget_get_events(inputbox)|GDK_FOCUS_CHANGE_MASK));
 
   gtk_signal_connect(GTK_OBJECT(inputbox),"focus_out_event",
-		     GTK_SIGNAL_FUNC(mainwindow_gcp_about_to_get_angry),0);
+		     GTK_SIGNAL_FUNC(mainwindow_input_focus_out),NULL);
 
   ims=new InputModeSelector();
 
@@ -298,7 +310,7 @@ MainWindow::MainWindow() {
 
   gshow(inputbox);
   ims->show();
-  Gtk::show(tw[0],tw[1],0);
+  Gtk::show(tw[0],tw[1],NULL);
   gtk_box_pack_start(GTK_BOX(v),tw[0],FALSE,FALSE,0);
 
   InputHistory=new History(256);
@@ -396,10 +408,7 @@ MainWindow::MainWindow() {
   notebook->setListener(this);
   paneChanged(0,-1);
 
-  gtk_timeout_add(150,keep_gcp_cool,(gpointer)inputbox);
-
-  if (!global.Desk.wMain.isNull())
-    gtk_window_set_default_size(GTK_WINDOW(widget),-1,-1);
+  gtk_timeout_add(150,forced_focus,(gpointer)inputbox);
 }
 
 void MainWindow::update() {
@@ -450,7 +459,7 @@ void MainWindow::createSealPix(GtkWidget *box) {
   gtk_container_add(GTK_CONTAINER(fr),picseal);
 
   gtk_box_pack_start(GTK_BOX(box),fr,FALSE,FALSE,0);
-  Gtk::show(picseal,fr,0);
+  Gtk::show(picseal,fr,NULL);
 }
 
 void MainWindow::createNavbar(GtkWidget *box) {
@@ -505,7 +514,7 @@ void MainWindow::createNavbar(GtkWidget *box) {
 
 
   gtk_box_pack_end(GTK_BOX(box),fr,FALSE,FALSE,0);
-  Gtk::show(mb,lb,fr,0);
+  Gtk::show(mb,lb,fr,NULL);
 
   gtk_tooltips_set_tip(tooltips,b[0],_("goes back to start of game"),0);
   gtk_tooltips_set_tip(tooltips,b[1],_("goes back 1 halfmove"),0);
@@ -539,7 +548,7 @@ void MainWindow::setTitle(char *msg) {
 }
 
 void MainWindow::restoreDesk() {
-  restorePosition(& global.Desk.wMain);
+  // main window is restored in the constructor
 
   if (! global.Desk.wGames.isNull() ) {
     openGameList();
@@ -556,12 +565,7 @@ void MainWindow::restoreDesk() {
     adlist->restorePosition(& global.Desk.wAds);
   }
 
-  if (global.Desk.PanePosition)
-    gtk_paned_set_position(GTK_PANED(global.mainpaned),
-			   global.Desk.PanePosition);
-
   global.Desk.spawnConsoles(icsout);
-
 }
 
 void MainWindow::parseThemeFile(char *name) {
@@ -609,7 +613,7 @@ void MainWindow::parseThemeFile(char *name) {
       }
       if ( (*it)->isNameDupe(te) ) {
 	++ndc;
-	sprintf(aux," (%d)",ndc);
+	snprintf(aux,16," (%d)",ndc);
 	te->Text+=aux;
       }
     }
@@ -641,21 +645,20 @@ void MainWindow::updateBookmarks() {
   i=1;
   for(hi=global.HostHistory.begin();hi!=global.HostHistory.end();hi++,i++) {
     snprintf(z,256,_("%d. Connect to %s:%d (%s)"),i,(*hi)->host,(*hi)->port,(*hi)->protocol);
-    z[255]=0;
     item=gtk_menu_item_new_with_label( z );
 
     gtk_signal_connect(GTK_OBJECT(item),"activate",
 		       GtkSignalFunc(mainwindow_connect_bookmark),
 		       (gpointer)(*hi));
 
-    gtk_menu_append(GTK_MENU(pmenu),item);
+    gtk_menu_shell_append(GTK_MENU_SHELL(pmenu),item);
     gshow(item);    
   }
 
   if (global.HostHistory.empty()) {
     item=gtk_menu_item_new_with_label(_("(no bookmarks)"));
     gtk_widget_set_sensitive(item,FALSE);
-    gtk_menu_append(GTK_MENU(pmenu),item);
+    gtk_menu_shell_append(GTK_MENU_SHELL(pmenu),item);
     gshow(item);    
   }
 
@@ -670,21 +673,21 @@ void MainWindow::updateBookmarks() {
 
   if (!global.EnginePresets.empty()) {
     item=gtk_menu_item_new_with_label(_("Edit Bookmarks..."));
-    gtk_menu_append(GTK_MENU(pmenu),item);
+    gtk_menu_shell_append(GTK_MENU_SHELL(pmenu),item);
     gshow(item);
 
     gtk_signal_connect(GTK_OBJECT(item),"activate",
 		       GtkSignalFunc(mainwindow_edit_engbm),
 		       (gpointer) this);
 
-    item=gtk_menu_item_new();
-    gtk_menu_append(GTK_MENU(pmenu),item);
+    item=gtk_separator_menu_item_new();
+    gtk_menu_shell_append(GTK_MENU_SHELL(pmenu),item);
     gshow(item);
   }
 
   i=1;
   for(ei=global.EnginePresets.begin();ei!=global.EnginePresets.end();ei++, i++) {
-    sprintf(z,"%d. ",i);
+    snprintf(z,256,"%d. ",i);
     x=z;
     x+=(*ei)->caption;
     item=gtk_menu_item_new_with_label( x.c_str() );
@@ -693,14 +696,14 @@ void MainWindow::updateBookmarks() {
 		       GtkSignalFunc(mainwindow_connect_bookmark2),
 		       (gpointer)(*ei));
 
-    gtk_menu_append(GTK_MENU(pmenu),item);
+    gtk_menu_shell_append(GTK_MENU_SHELL(pmenu),item);
     gshow(item);    
   }
 
   if (global.EnginePresets.empty()) {
     item=gtk_menu_item_new_with_label(_("(no bookmarks)"));
     gtk_widget_set_sensitive(item,FALSE);
-    gtk_menu_append(GTK_MENU(pmenu),item);
+    gtk_menu_shell_append(GTK_MENU_SHELL(pmenu),item);
     gshow(item);    
   }
 }
@@ -724,9 +727,9 @@ void MainWindow::searchThemes() {
   j=eff.getPathCount();
 
   for(i=0;i<j;i++) {
-    strcpy(tmp,eff.getPath(i).c_str());
-    strcat(tmp,"/");
-    strcat(tmp,"eboard_themes.conf");
+    g_strlcpy(tmp,eff.getPath(i).c_str(),512);
+    g_strlcat(tmp,"/",512);
+    g_strlcat(tmp,"eboard_themes.conf",512);
     parseThemeFile(tmp);
   }
 
@@ -737,7 +740,7 @@ void MainWindow::searchThemes() {
     
     while( (ds=readdir(dh)) != 0 ) {
       if (ExtraConf.match(ds->d_name)) {
-	sprintf(tmp,DATADIR "/eboard/%s",ds->d_name);
+	snprintf(tmp,512,DATADIR "/eboard/%s",ds->d_name);
 	parseThemeFile(tmp);
       }
     }
@@ -753,7 +756,7 @@ void MainWindow::searchThemes() {
       gtk_signal_connect(GTK_OBJECT(menuitem),"activate",
 			 GtkSignalFunc(mainwindow_themeitem),
 			 (gpointer)(*it));
-      gtk_menu_append(GTK_MENU(tmenu[0]),menuitem);
+      gtk_menu_shell_append(GTK_MENU_SHELL(tmenu[0]),menuitem);
       gshow(menuitem);
 
       // load pieces
@@ -762,7 +765,7 @@ void MainWindow::searchThemes() {
       gtk_signal_connect(GTK_OBJECT(menuitem),"activate",
 			 GtkSignalFunc(mainwindow_themeitem2),
 			 (gpointer)(*it));
-      gtk_menu_append(GTK_MENU(tmenu[1]),menuitem);
+      gtk_menu_shell_append(GTK_MENU_SHELL(tmenu[1]),menuitem);
       gshow(menuitem);
 
       // load squares
@@ -771,7 +774,7 @@ void MainWindow::searchThemes() {
       gtk_signal_connect(GTK_OBJECT(menuitem),"activate",
 			 GtkSignalFunc(mainwindow_themeitem3),
 			 (gpointer)(*it));
-      gtk_menu_append(GTK_MENU(tmenu[2]),menuitem);
+      gtk_menu_shell_append(GTK_MENU_SHELL(tmenu[2]),menuitem);
       gshow(menuitem);
 
     }
@@ -784,9 +787,9 @@ void MainWindow::searchThemes() {
 
 void MainWindow::greet() {
   char z[128];
-  sprintf (z,_("eboard version %s (%s)"),global.Version,global.SystemType);
+  snprintf (z,128,_("eboard version %s (%s)"),global.Version,global.SystemType);
   icsout->append(z,0xc0ff00,IM_IGNORE);
-  sprintf (z,_("(c) 2000-%d Felipe Bergo <bergo@seul.org> (FICS handle: Pulga)"), 2003);
+  snprintf (z,128,_("(c) 2000-%d Felipe Bergo <fbergo@gmail.com> (FICS handle: Pulga)"), 2007);
   icsout->append(z,0xc0ff00,IM_IGNORE);
   icsout->append(_("Distributed under the terms of the GNU General Public License, version 2 or later"),0xffc000,IM_IGNORE);
   icsout->append("http://www.gnu.org/copyleft/gpl.html",0xffc000,IM_IGNORE);
@@ -868,7 +871,7 @@ void MainWindow::consoleClosed() {
   consolecopy=0;
 }
 
-void MainWindow::userInput(char *text) {
+void MainWindow::userInput(const char *text) {
   char *nv;
   int i,j;
   if (text==0) return;
@@ -899,7 +902,14 @@ void MainWindow::injectInput() {
   char z[4096],y[256];
   int id;
 
-  strcpy(z,gtk_entry_get_text(GTK_ENTRY(inputbox)));
+  g_strlcpy(z,gtk_entry_get_text(GTK_ENTRY(inputbox)),4096);
+
+  if (ims->getSearchMode()) {
+    inconsole->SearchString = z;
+    inconsole->execSearch();
+    ims->setSearchMode(false);
+    goto ii_nothing_else;
+  }
 
   if (asetprefix.match(z)) {
     id=notebook->getCurrentPageId();
@@ -909,20 +919,19 @@ void MainWindow::injectInput() {
   }
 
   if (arunscript.match(z)) {
-    strncpy(y,arunscript.getStarToken(0),255); 
-    y[255]=0;
+    g_strlcpy(y,arunscript.getStarToken(0),256); 
     new ScriptInstance(y);
     goto ii_nothing_else;
     return;
   }
 
   if (ims->getChatMode()) {
-    strcpy(z,ims->getPrefix().c_str());
+    g_strlcpy(z,ims->getPrefix().c_str(),4096);
     if (z[0])
-      strcat(z," ");
+      g_strlcat(z," ",4096);
   } else
     z[0]=0;
-  strcat(z,gtk_entry_get_text(GTK_ENTRY(inputbox)));
+  g_strlcat(z,gtk_entry_get_text(GTK_ENTRY(inputbox)),4096);
 
   userInput(z);
 
@@ -932,10 +941,12 @@ void MainWindow::injectInput() {
 
 void MainWindow::historyUp() {
   gtk_entry_set_text(GTK_ENTRY(inputbox),InputHistory->moveUp());
+  gtk_editable_set_position(GTK_EDITABLE(inputbox),-1);
 }
 
 void MainWindow::historyDown() {
   gtk_entry_set_text(GTK_ENTRY(inputbox),InputHistory->moveDown());
+  gtk_editable_set_position(GTK_EDITABLE(inputbox),-1);
 }
 
 void MainWindow::saveBuffer() {
@@ -951,8 +962,7 @@ void MainWindow::saveDesk() {
   if (stocklist)  global.Desk.wLocal.retrieve(stocklist->widget);
   if (scriptlist) global.Desk.wLocal.retrieve(scriptlist->widget);
 
-  global.Desk.PanePosition = GTK_PANED(global.mainpaned)->child1_size;
-
+  global.Desk.PanePosition = gtk_paned_get_position(GTK_PANED(global.mainpaned));
   global.gatherConsoleState();
   global.writeRC();  
 }
@@ -961,7 +971,6 @@ void MainWindow::openServer(char *host,int port,Protocol *protocol,
 			    char *helper) {
   global.debug("MainWindow","openServer",host);
   inconsole->pop();
-  gtk_widget_grab_focus(inputbox);
   tryConnect(host,port,protocol,helper);
 }
 
@@ -970,7 +979,6 @@ void MainWindow::openServer(NetConnection *conn, Protocol *protocol) {
     return;
 
   inconsole->pop();
-  gtk_widget_grab_focus(inputbox);
 
   if ((global.protocol)||(global.network))
     disconnect();
@@ -1081,8 +1089,8 @@ void MainWindow::tryConnect(char *host,int port,Protocol *protocol,
 
   if (net->open()) {
     // failure
-    global.protocol=0;
-    global.network=0;
+    global.protocol=NULL;
+    global.network=NULL;
     status->setText(net->getError(),10);
     delete net;
     delete protocol;
@@ -1104,15 +1112,15 @@ void MainWindow::cleanUpConnection() {
   global.debug("MainWindow","cleanUpConnection");
   setSealPix(0);
 
-  if (global.protocol) {
+  if (global.protocol!=NULL) {
     global.protocol->finalize();
     delete global.protocol;
-    global.protocol=0;
+    global.protocol=NULL;
   }
-  if (global.network) {
+  if (global.network!=NULL) {
     global.network->close();
     delete global.network;
-    global.network=0;    
+    global.network=NULL;    
     icsout->append(_("--- Disconnected"),0xc0ff00);
   }
 
@@ -1140,7 +1148,7 @@ void MainWindow::cloneOnScratch(ChessGame *cg0) {
   cg->setBoard(b);
   cg->StopClock = 1;
 
-  sprintf(z,_("Scratch %d"),++global.LastScratch);
+  snprintf(z,64,_("Scratch %d"),++global.LastScratch);
   notebook->addPage(b->widget,z,id);
   b->setNotebook(notebook,id);
   notebook->goToPageId(id);
@@ -1165,7 +1173,7 @@ void MainWindow::newScratchBoard(bool clearboard) {
   cg->setBoard(b);
   cg->StopClock = 1;
 
-  sprintf(z,_("Scratch %d"),++global.LastScratch);
+  snprintf(z,64,_("Scratch %d"),++global.LastScratch);
   notebook->addPage(b->widget,z,id);
   b->setNotebook(notebook,id);
 
@@ -1175,7 +1183,7 @@ void MainWindow::newScratchBoard(bool clearboard) {
       for(j=0;j<8;j++)
 	p->setPiece(i,j,EMPTY);
 
-  cg->updatePosition(*p,1,0,0,0,_("<editing>"),false);
+  cg->updatePosition2(*p,1,0,0,0,_("<editing>"),false);
   notebook->goToPageId(id);
 }
 
@@ -1226,11 +1234,11 @@ void MainWindow::gameWalk(int op) {
       global.ebook->removePage(cg->GameNumber);
       global.removeBoard(b);
       delete(b);
-      cg->setBoard(0);
+      cg->setBoard(NULL);
       if (cg->LocalEdit)
 	global.deleteGame(cg);
     } else {
-      if ((id>0)&&(global.protocol))
+      if ((id>0)&&(global.protocol!=NULL))
 	global.protocol->discardGame(cg->GameNumber);
     }
     break;
@@ -1266,6 +1274,53 @@ gboolean do_smart_remove(gpointer data) {
   return FALSE;
 }
 
+void MainWindow::joystickEvent(JoystickEventType jet, int number, int value) {
+  int id, anum;
+  ChessGame *cg = NULL;
+  Board *b = NULL;
+
+  if (global.joycapture != NULL) {
+    global.joycapture->joystickEvent(jet,number,value);
+    return;
+  }
+  
+  id=notebook->getCurrentPageId();
+
+  if (id==-1) 
+    b=global.BoardList.front();
+  else {
+    cg=global.getGame(id);
+    if (cg!=NULL)
+      b=cg->getBoard();
+  }
+
+  anum = number & 0xfffe;
+
+  if (jet == JOY_AXIS) {
+    if (anum == global.JSCursorAxis && b!=NULL && 
+	(global.JSMode==1 || value<-16000 || value>16000 || value==0) )
+      b->joystickCursor(number%2, value);
+    if (number == global.JSBrowseAxis && b!=NULL && (value<-16000 || value>16000 || value==0) ) {
+      if (value < 0) value = -1;
+      if (value > 0) value =  1;
+
+      if (value < 0 && jpd!=value) b->walkBack1();
+      if (value > 0 && jpd!=value) b->walkForward1();
+      jpd = value;
+    }
+  }
+
+  if (jet == JOY_BUTTON && value==1) {
+    if (number == global.JSMoveButton && b!=NULL)
+      b->joystickSelect();
+    if (number == global.JSPrevTabButton)
+      global.ebook->goToPrevious();
+    if (number == global.JSNextTabButton)
+      global.ebook->goToNext();
+  }
+
+}
+
 // changes enabling of buttons upon pane change
 // also schedules 'smart removes' for finished boards
 void MainWindow::paneChanged(int pgseq,int pgid) {
@@ -1293,7 +1348,7 @@ void MainWindow::paneChanged(int pgseq,int pgid) {
     int ch;
     char z[32];
     ch=-200-pgid;
-    sprintf(z,"t %d",ch);
+    snprintf(z,32,"t %d",ch);
     ims->setPrefix(z);
     imscache.set(pgid, ims->getPrefix());
   } else if (pgid >= 0) {
@@ -1368,11 +1423,9 @@ main_key_press (GtkWidget * wid, GdkEventKey * evt, gpointer data) {
     switch(evt->keyval) {
     case GDK_Left:
       ptr->gameWalk(1);
-      gtk_widget_grab_focus(ptr->inputbox);
       break;
     case GDK_Right:
       ptr->gameWalk(2);
-      gtk_widget_grab_focus(ptr->inputbox);
       break;
     case GDK_F:
     case GDK_f:
@@ -1395,23 +1448,18 @@ main_key_press (GtkWidget * wid, GdkEventKey * evt, gpointer data) {
     break;
   case GDK_F3: // previous pane
     global.ebook->goToPrevious();
-    gtk_widget_grab_focus(ptr->inputbox);
     break;
   case GDK_F4: // next pane
     global.ebook->goToNext();
-    gtk_widget_grab_focus(ptr->inputbox);
     break;
   case GDK_F5: // pop main board
     global.ebook->goToPageId(-1);
-    gtk_widget_grab_focus(ptr->inputbox);
     break;
   case GDK_F6: // pop console
     global.ebook->goToPageId(-2);
-    gtk_widget_grab_focus(ptr->inputbox);
     break;
   case GDK_F7: // pop seek graph
     global.ebook->goToPageId(-3);
-    gtk_widget_grab_focus(ptr->inputbox);
     break;
   case GDK_F8: // toggle shortcut bar
     global.ShowQuickbar = ! (global.ShowQuickbar);    
@@ -1588,7 +1636,7 @@ windows_savebuffer(GtkWidget *w, gpointer data) {
 
 void
 windows_find(GtkWidget *w, gpointer data) {
-  mainw->inconsole->findText();
+  mainw->ims->setSearchMode(true);
 }
 
 void
@@ -1624,7 +1672,7 @@ void MainWindow::readAvailable(int handle) {
   global.debug("MainWindow","readAvailable");
   net = global.network;
 
-  if ((net==NULL)||(!net->isConnected())||(!global.protocol)) {
+  if ((net==NULL)||(!net->isConnected())||(global.protocol==NULL)) {
     cleanUpConnection();
     return;
   }
@@ -1670,7 +1718,7 @@ gboolean mainwindow_read_agents(gpointer data) {
   int loopc;
   char line[2048];
 
-  if (!global.protocol)
+  if (global.protocol==NULL)
     return FALSE;
 
   loopc = 0;
@@ -1852,17 +1900,20 @@ void game_retract(GtkWidget *w,gpointer data) {
 // kept there...
 
 gboolean
-keep_gcp_cool(gpointer data)
+forced_focus(gpointer data)
 {
+  gint a,b;
+  gtk_editable_get_selection_bounds(GTK_EDITABLE(data),&a,&b); //FIXME
   gtk_widget_grab_focus(GTK_WIDGET(data));
+  gtk_editable_select_region(GTK_EDITABLE(data),a,b);
   return FALSE;
 }
 
 gboolean
-mainwindow_gcp_about_to_get_angry(GtkWidget *widget,GdkEventFocus *event,gpointer user_data)
+mainwindow_input_focus_out(GtkWidget *widget,GdkEventFocus *event,gpointer user_data)
 {
-  gtk_timeout_add(100,keep_gcp_cool,(gpointer)widget);
-  return TRUE;
+  gtk_timeout_add(100,forced_focus,(gpointer)widget);
+  return FALSE;
 }
 
 void
@@ -1913,6 +1964,7 @@ InputModeSelector::InputModeSelector() {
   int i;
 
   ChatMode=false;
+  SearchMode=false;
   prefix.erase();
 
   widget = gtk_button_new();
@@ -1937,6 +1989,25 @@ bool InputModeSelector::getChatMode() {
   return ChatMode;
 }
 
+bool InputModeSelector::getSearchMode() {
+  return SearchMode;
+
+}
+
+void InputModeSelector::setSearchMode(bool m) {
+  SearchMode = m;
+
+  if (!SearchMode) {
+    setChatMode(ChatMode);
+    return;
+  }
+
+  gtk_label_set_text(GTK_LABEL(l[0]), _("[find]"));
+  gtk_label_set_text(GTK_LABEL(l[1]), "");
+  setColor(l[0],0,0x80,0);
+  setColor(l[1],0,0,0);
+}
+
 void InputModeSelector::setChatMode(bool m) {
   string x;
   ChatMode=m;
@@ -2034,3 +2105,16 @@ void PrefixCache::setIfNotSet(int id, string &val) {
   ids.push_back(id);
   text.push_back( new string(val) );
 }
+
+#ifdef HAVE_LINUX_JOYSTICK_H
+void mainwindow_joystick(gpointer data,gint source,GdkInputCondition cond) {
+  struct js_event jse;
+  MainWindow *me = (MainWindow *) data;
+  if (read(global.JoystickFD,&jse,sizeof(struct js_event)) == sizeof(struct js_event)) {
+    switch(jse.type) {
+    case JS_EVENT_BUTTON: me->joystickEvent(JOY_BUTTON, (int) (jse.number), (int) (jse.value)); break;
+    case JS_EVENT_AXIS:   me->joystickEvent(JOY_AXIS, (int) (jse.number), (int) (jse.value)); break;      
+    }
+  }
+}
+#endif
