@@ -1,10 +1,10 @@
-/* $Id: movelist.cc,v 1.15 2003/01/19 19:24:02 bergo Exp $ */
+/* $Id: movelist.cc,v 1.18 2007/01/20 15:58:43 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -44,9 +44,9 @@ MoveListWindow::MoveListWindow(char *p1,char *p2,
   listener=0;
 
   if (gid<7000)  
-    sprintf(z,_("Game #%d - %s vs. %s"),gid,p1,p2);
+    snprintf(z,128,_("Game #%d - %s vs. %s"),gid,p1,p2);
   else
-    sprintf(z,"%s vs. %s",p1,p2);
+    snprintf(z,128,"%s vs. %s",p1,p2);
   
   widget=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_default_size(GTK_WINDOW(widget),300,400);
@@ -82,7 +82,7 @@ MoveListWindow::MoveListWindow(char *p1,char *p2,
 
   populate_clist(moves,over,result,reason);
 
-  Gtk::show(clist,sw,v,0);
+  Gtk::show(clist,sw,v,NULL);
 
   gtk_signal_connect (GTK_OBJECT (widget), "destroy",
 		      GTK_SIGNAL_FUNC (movelist_destroy), this);
@@ -122,7 +122,7 @@ void MoveListWindow::populate_clist(list<Position> &moves, int over,
       tm=0;
     if (!tm) continue;
     if (tm!=cm) {
-      sprintf(z,"%d",tm);
+      snprintf(z,64,"%d",tm);
       zz[0]=z;
       zz[1]=_("none");
       zz[2]=_("none");
@@ -151,7 +151,7 @@ void MoveListWindow::populate_clist(list<Position> &moves, int over,
     case UNDEF:     strcpy(z,"(*) "); break;
     default: z[0]=0;
     }
-    strcat(z,reason);
+    g_strlcat(z,reason,64);
     textbar->setText(z);
   } else {
     textbar->setText(_("Game in progress."));
