@@ -1,10 +1,10 @@
-/* $Id: network.h,v 1.18 2003/07/03 19:47:53 bergo Exp $ */
+/* $Id: network.h,v 1.22 2007/01/03 17:48:53 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -40,7 +40,13 @@
 #include <sys/socket.h>
 #include <netdb.h>
 #include <netinet/in.h>
+
+#include "config.h"
+
+#ifdef NEED_TCP_H
 #include <netinet/tcp.h>
+#endif
+
 #include <arpa/inet.h>
 
 #include <gtk/gtk.h>
@@ -106,6 +112,8 @@ class NetConnection {
   virtual void notifyReadReady(IONotificationInterface *target);
   virtual void sendReadNotify();
 
+  void cleanUp();
+
  private:
   int  TagRead;
   IONotificationInterface *listener;
@@ -114,6 +122,9 @@ class NetConnection {
 				  GdkInputCondition cond);
 };
 
+void netconn_read_notify(gpointer data, gint source, 
+			 GdkInputCondition cond);
+
 class BufferedConnection : public NetConnection {
  public:
   virtual int readPartial(char *tbuffer,int limit);
@@ -207,6 +218,7 @@ class PipeConnection : public BufferedConnection,
   int   getReadHandle();
 
   virtual void farewellPid(int dpid);
+  void setMaxWaitTime(double msecs);
 
  private:
   void init();
@@ -223,8 +235,11 @@ class PipeConnection : public BufferedConnection,
   int  pid;
   int  toid; // timeout
   string handshake;
+  double MaxWaitTime; // msecs
 };
 
+gboolean sched_close(gpointer data);
+
 class FallBackConnection : public NetConnection {
  public:
   FallBackConnection();
