@@ -1,10 +1,10 @@
-/* $Id: position.cc,v 1.40 2002/06/14 17:32:57 bergo Exp $ */
+/* $Id: position.cc,v 1.61 2008/02/18 13:21:05 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 */
 
 
-#include <iostream.h>
+#include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -32,6 +32,7 @@
 #include <math.h>
 #include "position.h"
 #include "global.h"
+#include "eboard.h"
 
 StringCollection::StringCollection() {
   amOpen = false;
@@ -56,6 +57,11 @@ void StringCollection::append(char *s) {
     collection.back().s.append(s);
 }
 
+void StringCollection::append(string &s) {
+  if (amOpen)
+    collection.back().s.append(s);
+}
+
 void StringCollection::append(char c) {
   if (amOpen)
     collection.back().s.append(1,c);
@@ -124,12 +130,10 @@ char Position::AnnotationBuffer[512];
 Position::Position() {
   setStartPos();
   LastMove.erase();
-  global.create("Position");
 }
 
 Position::~Position() {
   dropAnnotations();
-  global.destroy("Position");
 }
 
 void Position::dropAnnotations() {
@@ -163,7 +167,7 @@ void Position::setStartPos() {
   sidehint=true;
 }
 
-int Position::operator==(Position &p) {
+int Position::operator==(const Position &p) {
   int i,j;
   for(i=0;i<8;i++)
     for(j=0;j<8;j++)
@@ -176,8 +180,8 @@ int Position::operator==(Position &p) {
   return 1;
 }
 
-int Position::operator!=(Position &p) {
-  return(!(p==(*this)));
+int Position::operator!=(const Position &p) {
+  return(!((*this)==p));
 }
 
 Position Position::operator=(Position &p) {
@@ -238,7 +242,6 @@ void Position::clearAnnotation() {
 }
 
 void Position::invalidate() {
-  piece c;
   int i,j;
   for(i=0;i<8;i++)
     for(j=0;j<8;j++)
@@ -257,7 +260,7 @@ string & Position::getLastMove() {
   return(LastMove);
 }
 
-void Position::moveAnyNotation(char *m,piece color,variant Vr=REGULAR) {
+void Position::moveAnyNotation(char *m,piece color,variant Vr) {
   int i,x,y,ml;
   piece pr;
   char xlate[12];
@@ -289,11 +292,11 @@ void Position::moveAnyNotation(char *m,piece color,variant Vr=REGULAR) {
   ml = strlen(m);
 
   if ( isupper(m[0]) || m[1]=='x' || m[2]=='x' || m[0]=='o' || ml==2 || 
-       ml == 3 || m[2]=='=' || ( ml >= 4 && ( m[3]=='+' || m[3]=='#' ) ) 
-       ) 
+       ml == 3 || m[2]=='=' || ( ml >= 4 && ( m[3]=='+' || m[3]=='#' ) )
+       || (ml>=5 && m[4]=='=') ) 
     {
       if (m[0]=='o') { // cope with o-o for castling (gnu chess 4)
-	strcpy(xlate,m);
+	g_strlcpy(xlate,m,12);
 	for(i=0;xlate[i];i++)
 	  xlate[i]=toupper(xlate[i]);
 	moveStdNotation(xlate,color,Vr);
@@ -317,7 +320,7 @@ void Position::moveAnyNotation(char *m,piece color,variant Vr=REGULAR) {
     }
 }
 
-void Position::moveStdNotation(char *m,piece color,variant Vr=REGULAR) {
+void Position::moveStdNotation(char *m,piece color,variant Vr) {
   int from[2],to[2];
   int istake, isdrop;
   piece what, prom;
@@ -430,8 +433,12 @@ void Position::moveStdNotation(char *m,piece color,variant Vr=REGULAR) {
   }
 
   // check whether kings and rooks moved
-  if ((Vr!=WILD)&&(Vr!=SUICIDE))
+  if (IS_NOT_WILD(Vr) && (Vr!=SUICIDE))
     checkCastlingPossibility();
+
+  // handle explosions in atomic
+  if (Vr==ATOMIC && istake)
+    makeAtomicExplosion(to[0],to[1]);
 }
 
 void Position::checkCastlingPossibility() {
@@ -449,7 +456,7 @@ void Position::checkCastlingPossibility() {
 }
 
 void Position::locate(piece p,int *src,int *dest,int istake,
-		      variant Vr=REGULAR) {
+		      variant Vr) {
   int minx,maxx,miny,maxy;
   int i,j,dc,dr,m,n;
   piece kind,color;
@@ -563,10 +570,11 @@ void Position::moveDrop(piece p,int x2,int y2) {
 }
 
 void Position::moveCartesian(int x1,int y1,int x2,int y2,
-			     variant Vr=REGULAR, 
-			     bool resolvepromotion=false)
+			     variant Vr, 
+			     bool resolvepromotion)
 {
   piece color;
+  int   istake=0;
 
   color=square[x1][y1]&COLOR_MASK;
   if (color==WHITE) sidehint=false; else sidehint=true;
@@ -588,6 +596,7 @@ void Position::moveCartesian(int x1,int y1,int x2,int y2,
       (x1!=x2)) {
     square[x2][y2+((color==WHITE)?-1:1)]=EMPTY;
     if (ep[0]>=0) ep[0]=-1;
+    istake = 1;
   }
 
   if ((Vr==BUGHOUSE)||(Vr==CRAZYHOUSE))
@@ -602,6 +611,8 @@ void Position::moveCartesian(int x1,int y1,int x2,int y2,
     if (ep[0]>=0) ep[0]=-1;
   }
 
+  if (square[x2][y2] != EMPTY) istake=1;
+
   square[x2][y2]=square[x1][y1];
   square[x1][y1]=EMPTY;
 
@@ -620,8 +631,22 @@ void Position::moveCartesian(int x1,int y1,int x2,int y2,
       }      
   }
 
-  if ((Vr!=WILD)&&(Vr!=SUICIDE))
+  if (IS_NOT_WILD(Vr) && (Vr!=SUICIDE))
     checkCastlingPossibility();
+
+  // handle explosions in atomic
+  if (Vr==ATOMIC && istake)
+    makeAtomicExplosion(x2,y2);
+}
+
+/* does not test variant or if a capture actually occurred */
+void Position::makeAtomicExplosion(int x,int y) {
+  int i,j;
+  square[x][y] = EMPTY;
+  for(i=x-1;i<=x+1;i++) if (i>=0 && i<=7)
+    for(j=y-1;j<=y+1;j++) if (j>=0 && j<=7)
+      if ( (square[i][j] & PIECE_MASK) != PAWN)
+	square[i][j] = EMPTY;
 }
 
 void Position::SANstring(char *src,char *dest) {
@@ -642,7 +667,8 @@ void Position::SANstring(char *src,char *dest) {
 
   // already SAN, just capitalize castling if needed
   if ( (isupper(src[0])) || (src[1]=='x') ||
-       (src[2]=='x') || (src[0]=='o') || (strlen(src)==2) || (src[2]=='=') ) {
+       (src[2]=='x') || (src[0]=='o') || (strlen(src)==2) || (src[2]=='=') ||
+       ((src[2]=='+' || src[2]=='#') && (strlen(src)==3)) ) {
     strcpy(dest,src);
     if (dest[0]=='o')
       for(i=0;dest[i];i++)
@@ -664,7 +690,7 @@ void Position::SANstring(char *src,char *dest) {
   stdNotationForMove(x1,y1,x2,y2,prom,dest);
 }
 
-void Position::stdNotationForMove(int x1,int y1,int x2,int y2,piece prom,char *m,variant Vr=REGULAR) {
+void Position::stdNotationForMove(int x1,int y1,int x2,int y2,piece prom,char *m,variant Vr) {
   Position after;
   stdNotationForMoveInternal(x1,y1,x2,y2,m);
   switch(prom) {
@@ -689,8 +715,8 @@ void Position::stdNotationForMove(int x1,int y1,int x2,int y2,piece prom,char *m
 
 void Position::stdNotationForMoveInternal(int x1,int y1,int x2,int y2,char *m) {
   vector<int> ax,ay;
-  int istake;
-  int hasamb,needrow,needcol,i;
+  int istake, hasamb,needrow,needcol;
+  unsigned int i;
   piece p,c;
   char *z;
 
@@ -894,61 +920,87 @@ void Position::print() {
     cerr << "\n+---+---+---+---+---+---+---+---+\n";
   }
   cerr << endl;
+
+  cerr << "stock = ";
+  for(i=0;i<2;i++)
+    for(j=0;j<5;j++)
+      cerr << House[j][i] << " ";
+  cerr << endl;
+
 }
 
+/* find the diff from current position to np */
 void Position::diff(Position &np,vector<SMove *> &sl) {
-  int i,j;
-  int pend,futile,futility;
-  int sieve[8][8];
+  bool sieve[64];
+  unsigned int ui;
+  int i,j,pend,pi=0,qi=0;
+  int sx=0,sy=0,dx=0,dy=0;
+  piece p;
 
-  for(i=0;i<sl.size();i++)
-    delete(sl[i]);
+  // clear output vector
+  for(ui=0;ui<sl.size();ui++)
+    delete(sl[ui]);
   sl.clear();
-
+  
+  // set sieve
+  pend = 0; // different squares left
   for(i=0;i<8;i++)
-    for(j=0;j<8;j++)
-      sieve[i][j]=0;
+    for(j=0;j<8;j++) {
+      sieve[i*8+j] = ( getPiece(i,j) != np.getPiece(i,j) );
+      if (sieve[i*8+j]) ++pend;
+    }
 
-  pend=0;
-  futility=0;
-  do {
-    futile=1;
-    for(i=0;i<8;i++)
-      for(j=0;j<8;j++) {
-	if (square[i][j]==np.square[i][j])
-	  continue;
-	
-	if ((np.square[i][j]==EMPTY)&&(!sieve[i][j])) {
-	  sl.push_back(new SMove(square[i][j],i,j,0,0));
-	  sieve[i][j]=1;
-	  pend++;
-	  futile=0;
-	} else {
-	  if ((pend)&&(np.square[i][j] == sl.back()->Piece)) {
-	    sl.back()->DX=i;
-	    sl.back()->DY=j;
-	    pend--;
-	    futile=0;
-	  }
-	  // avoid infinite loop on promotions
-	  if ( (pend)&&((sl.back()->Piece|PIECE_MASK) == PAWN)&&
-	       (futility>=2)&&(!(j%7)) ) {
-	    sl.back()->DX=i;
-	    sl.back()->DY=j;
-	    pend--;
-	    futile=0;
-	  }
-	}
+  while(pend > 1) {
+    dx = -1;
+    for(i=0;i<64;i++)
+      if (sieve[i] && np.getPiece(i/8,i%8)!=EMPTY && getPiece(i/8,i%8)==EMPTY) {
+	dx = i/8; dy = i%8; pi = i;
+	break;
       }
-    if (futile)
-      ++futility;
-  } while((pend)&&(futility<5));
 
-  // the only situation I can think of is the taken pawn on
-  // an en-passant take
-  if (pend) {
-    delete(sl.back());
-    sl.pop_back();
+    // no plain moves left, consider captures
+    if (dx<0) {
+      for(i=0;i<64;i++)
+	if (sieve[i] && np.getPiece(i/8,i%8)!=EMPTY && getPiece(i/8,i%8)!=EMPTY) {
+	  dx = i/8; dy = i%8; pi = i;
+	  break;
+	}      
+    }    
+
+    if (dx<0)
+      return; // I don't know what to do, bail out. shouldn't ever happen
+
+    p = np.getPiece(dx,dy);
+
+    sx = -1;
+    for(i=0;i<64;i++)
+      if (sieve[i] && i!=pi && getPiece(i/8,i%8)==p) {
+	sx = i/8;  sy = i%8; qi = i;
+	break;
+      }
+
+    // promotion ? (WASPAWN can't be checked, FICS positions won't have it)
+    if (sx < 0 && ((p&WHITE && dy==7) || (p&BLACK && dy==0))) {
+      p = PAWN | (p&COLOR_MASK);
+      for(i=0;i<64;i++)
+	if (sieve[i] && i!=pi && getPiece(i/8,i%8)==p) {
+	  sx = i/8;  sy = i%8; qi = i;
+	  break;
+	}
+    }
+    
+    // unable to match (?,?)@this -> (dx,dy)@np
+    // (will happen on crazyhouse drops, for example)
+    if (sx < 0) {
+      sieve[pi] = false;
+      --pend;
+    } else {
+      sl.push_back(new SMove(p, sx,sy, dx,dy));
+      sieve[pi] = sieve[qi] = false;
+      pend -= 2;
+    }
+    // and since we always decrement pend, we can ensure that this
+    // loop finishes
   }
 }
 
@@ -1003,7 +1055,7 @@ string & Position::getHouseString() {
   return(HouseString);
 }
 
-string & Position::getMaterialString(variant Vr=REGULAR) {
+string & Position::getMaterialString(variant Vr) {
   int w=0,b=0,i,j,v;
   char tmp[64];
 
@@ -1032,7 +1084,7 @@ string & Position::getMaterialString(variant Vr=REGULAR) {
 	b+=v;
     }
 
-  sprintf(tmp,"Material: %d - %d",w,b);
+  snprintf(tmp,64,_("Material: %d - %d"),w,b);
   MaterialString=tmp;
   return(MaterialString);
 }
@@ -1113,7 +1165,7 @@ void Position::decStockCount(piece p) {
 bool Position::isMoveLegalCartesian(int x0,int y0,int x1,int y1,piece mycolor,variant vari)
 {
   Position after;
-  int dx,dy;
+  int dx;
 
   //  cerr << "mycolor = " << mycolor << endl;
 
@@ -1122,14 +1174,18 @@ bool Position::isMoveLegalCartesian(int x0,int y0,int x1,int y1,piece mycolor,va
   if ((square[x0][y0]&COLOR_MASK)!=mycolor) return false;
   if ((square[x1][y1]&COLOR_MASK)==mycolor) return false;
 
+  if (vari == ATOMIC)
+    return true;
+
   // castling
   if ((square[x0][y0]&PIECE_MASK)==KING) {
     dx=abs(x0-x1);
     if (dx>1) {
       if ((mycolor==WHITE)&&((y0!=y1)||(y0!=0))) return false;
       if ((mycolor==BLACK)&&((y0!=y1)||(y0!=7))) return false;
-      if (vari==WILD) return true; // wild castling accepted always
+      if (vari==WILD || vari==WILDFR || vari==WILDCASTLE) return true; // wild castling accepted always
       if (vari==SUICIDE) return false; // suicide can't castle
+      if (vari==WILDNOCASTLE) return false; // nocastle shuffle chess
       if (dx!=2) return false;
       if ((x0==4)&&(x1==6)&&(maycastle[mycolor==WHITE?0:1]))
 	if ((square[5][y0]==EMPTY)&&(square[6][y0]==EMPTY))
@@ -1146,7 +1202,8 @@ bool Position::isMoveLegalCartesian(int x0,int y0,int x1,int y1,piece mycolor,va
   }
 
   // not checked for these
-  if ((vari==SUICIDE)||(vari==GIVEAWAY)||(vari==LOSERS)) return true;
+  if ((vari==SUICIDE)||(vari==GIVEAWAY)||(vari==LOSERS))
+    return true;
   
   if (!canMove(square[x0][y0]&PIECE_MASK,mycolor,x0,y0,x1,y1))
     return false;
@@ -1179,7 +1236,7 @@ bool Position::isDropLegal(piece p,int x1,int y1,piece mycolor,variant vari)
 }
 
 // dc = defender color
-bool Position::isSquareInCheck(int x,int y,piece dc,variant Vr=REGULAR) {
+bool Position::isSquareInCheck(int x,int y,piece dc,variant Vr) {
   int i,j;
 
   //  cerr << "is square in check " << (char)('a'+x) << (y+1) << " ";
@@ -1208,8 +1265,8 @@ bool Position::isSquareInCheck(int x,int y,piece dc,variant Vr=REGULAR) {
   return false;
 }
 
-bool Position::isInCheck(piece c,variant Vr=REGULAR) {
-  int i,j,kx,ky;
+bool Position::isInCheck(piece c,variant Vr) {
+  int i,j,kx,ky=0;
 
   // FIXME: check rules for giveaway and losers variants
   if (Vr==SUICIDE)
@@ -1228,7 +1285,7 @@ bool Position::isInCheck(piece c,variant Vr=REGULAR) {
   return(isSquareInCheck(kx,ky,c,Vr));
 }
 
-bool Position::isMate(piece c,variant Vr=REGULAR) {
+bool Position::isMate(piece c,variant Vr) {
   int i,j,m,n;
   Position after;
 
@@ -1255,13 +1312,62 @@ bool Position::isMate(piece c,variant Vr=REGULAR) {
   return true;
 }
 
+bool Position::isStalemate(piece c,variant Vr) {
+  int i,j,m,n;
+  Position after;
+
+  if (Vr!=REGULAR)
+    return false;
+
+  if (isInCheck(c,Vr))
+    return false;
+
+  for(i=0;i<8;i++)
+    for(j=0;j<8;j++)
+      for(m=0;m<8;m++)
+	for(n=0;n<8;n++) {
+	  if ((m==i)&&(n==j)) continue;
+	  if ((square[i][j]&COLOR_MASK)!=c) continue;
+	  if ((square[m][n]&COLOR_MASK)==c) continue;
+	  if (!isMoveLegalCartesian(i,j,m,n,c,Vr)) continue;
+	  after=(*this);
+	  after.moveCartesian(i,j,m,n,Vr);
+	  if (!after.isInCheck(c,Vr))
+	    return false; /* found a legal move, no stalemate */
+	}
+  return true;
+}
+
+/* draw situations recognized : K vs K
+                                K vs KN
+                                K vs KB 
+*/
+bool Position::isNMDraw(variant Vr) {
+  int n,b;
+  int i,j;
+  piece p;
+
+  if (Vr!=REGULAR) return false;
+  
+  n=b=0;
+
+  for(i=0;i<8;i++) for(j=0;j<8;j++) {
+    p = square[i][j] & PIECE_MASK;
+    if (p==QUEEN || p==ROOK || p==PAWN) return false;
+    if (p==BISHOP) ++b;
+    if (p==KNIGHT) ++n;
+    if ( (b+n) > 1 ) return false;
+  }
+  return true;
+}
+
 // FEN
 
 string & Position::getFEN() {
   static string fen;
   int i,j,ec;
-  piece p,c;
-  char n;
+  piece p,c=EMPTY;
+  char n=0;
 
   fen="\0";
 
@@ -1315,7 +1421,7 @@ string & Position::getFEN() {
   if (!ec) fen+="-";
   fen+=" ";
   
-  if (ep[0]>=0) { fen+='a'+ep[0]; fen+='0'+ep[1]; }
+  if (ep[0]>=0) { fen+='a'+ep[0]; fen+='1'+ep[1]; }
   else { fen+='-'; }
   fen+=" ";
   
@@ -1396,6 +1502,6 @@ void Position::setFEN(const char *fen) {
   ep[0]=-1;
   if ((*p)!='-') {
     ep[0]=(*p++)-'a';
-    ep[1]=(*p)-'0';
+    ep[1]=(*p)-'1';
   }  
 }
