@@ -1,10 +1,10 @@
-/* $Id: proto_fics.h,v 1.42 2003/07/05 02:48:19 bergo Exp $ */
+/* $Id: proto_fics.h,v 1.50 2008/02/08 14:25:51 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -57,7 +57,7 @@ class FicsProtocol : public Protocol {
   void exaForward(int n);
   void exaBackward(int n);
 
-  void refreshSeeks();
+  void refreshSeeks(bool create);
 
   vector<string *> * getPlayerActions();
   vector<string *> * getGameActions();
@@ -75,7 +75,7 @@ class FicsProtocol : public Protocol {
   void parseStyle12(char *data);
   void gameOver(ExtPatternMatcher &pm, GameResult result);
   void updateGame(ExtPatternMatcher &pm);
-  void attachHouseStock();
+  void attachHouseStock(ExtPatternMatcher &pm, bool OnlyForBughouse);
 
   void clearMoveBuffer();
   void retrieve1(ExtPatternMatcher &pm);
@@ -106,9 +106,39 @@ class FicsProtocol : public Protocol {
 
   friend gboolean fics_allob(gpointer data);
 
-  int AuthenticationGone;
-  bool InitializationSent;
-  bool LectureBotSpecial;
+  /* private parsing state and parsing methods */
+
+  bool AuthGone;
+  bool InitSent;
+  bool LecBotSpecial;
+  bool LastWasStyle12;
+  bool LastWasPrompt;
+  bool UseMsecs;
+
+  bool RetrievingGameList;
+  bool RetrievingAdList;
+  int  RetrievingMoves;
+  int  PendingStatus;
+
+  int  LastChannel;
+  int  LastColor;
+ 
+  /* all parser* methods return true if all parsing has been done,
+     false if something must still be done.
+     (and I'm trying to avoid any false returns, making it all
+      functional-programming-happy)
+ */
+
+  bool parser1(char *T); /* prompt removal */
+  bool parser2(char *T); /* early login */
+  bool parser3(char *T); /* game events */
+  bool parser4(char *T); /* list retrieval */
+  bool parser5(char *T); /* server/game events */
+  bool parser6(char *T); /* game termination */
+  bool parser7(char *T); /* bughouse and other oddities */
+  bool parser8(char *T); /* chat */
+
+  bool doOutput(char *msg, int channel, bool personal, int msgcolor);
 
   PatternBinder Binder;
 
@@ -137,6 +167,7 @@ class FicsProtocol : public Protocol {
 
   ExtPatternMatcher Style12;
   ExtPatternMatcher Style12House;
+  ExtPatternMatcher Style12BadHouse;
 
   ExtPatternMatcher WhiteWon;
   ExtPatternMatcher BlackWon;
@@ -173,11 +204,15 @@ class FicsProtocol : public Protocol {
   ExtPatternMatcher PTell;
   ExtPatternMatcher GotPartnerGame;
 
+  ExtPatternMatcher ObsInfoLine;
+  ExtPatternMatcher MsSet;
+
   // seek graph
   ExtPatternMatcher SgClear;
   ExtPatternMatcher SgAdd;
   ExtPatternMatcher SgAdd2;
   ExtPatternMatcher SgRemove;
+  ExtPatternMatcher SgRemove2;
 
   ExtPatternMatcher AllObNone;
   ExtPatternMatcher AllObFirstLine;
@@ -185,22 +220,14 @@ class FicsProtocol : public Protocol {
   int               AllObState;
   char              AllObAcc[1024];
 
-  int LastWasStyle12;
-
   char nick[64];
-  int LastWasPrompt;
   piece style12table[256];
-  int LastColor;
-  int LastWasCTell;
-  int PendingStatus;
-  int RetrievingMoves;
-  int RetrievingGameList;
-  int RetrievingAdList;
 
   list<Position> MoveBuf;
   deque<int> MoveRetrievals;
   deque<int> AllObReqs;
   vector<string *> PActions, GActions;
+  Position WildStartPos;
 
   GameListConsumer *lastGLC, *lastALC;
 
@@ -213,4 +240,6 @@ class FicsProtocol : public Protocol {
   bool UserPlayingWithWhite; /* for won/lost sound event tracking */
 };
 
+gboolean fics_allob(gpointer data);
+
 #endif
