@@ -1,10 +1,10 @@
-/* $Id: proto_p2p.cc,v 1.4 2003/07/02 23:50:24 bergo Exp $ */
+/* $Id: proto_p2p.cc,v 1.12 2008/02/08 14:25:51 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -48,7 +48,7 @@ P2PProtocol::P2PProtocol() {
 
   Binder.add(&Chat, &ProtoGreet, &NameGreet, 
 	     &GameProp, &Accept, &Move, &Outcome, 
-	     &DrawOffer, 0);
+	     &DrawOffer, NULL);
 
   IdSent = IdReceived = false;
 
@@ -68,6 +68,10 @@ P2PProtocol::~P2PProtocol() {
   tmpbuf = 0;
 }
 
+bool P2PProtocol::requiresLegalityChecking() {
+  return true;
+}
+
 void P2PProtocol::finalize() {
   if (pad) {
     pad->release();
@@ -84,9 +88,9 @@ void P2PProtocol::sendId() {
   if (!IdSent)
     if (global.network)
       if (global.network->isConnected()) {
-	sprintf(z,"P-(EBOARDP2P)-(eboard %s)-(1)",global.Version);
+	snprintf(z,128,"P-(EBOARDP2P)-(eboard %s)-(1)",global.Version);
 	global.network->writeLine(z);	
-	sprintf(z,"N-(%s)",global.P2PName);
+	snprintf(z,128,"N-(%s)",global.P2PName);
 	global.network->writeLine(z);
 	IdSent = true;
       }
@@ -96,9 +100,9 @@ void P2PProtocol::sendProposal(GameProposal &g) {
   char z[128];
   if (global.network)
     if (global.network->isConnected()) {
-      sprintf(z,"G-(%d)-(%d)-(%d)-(%s)",
-	      g.AmWhite ? 0 : 1,
-	      g.STime, g.SInc, ChessGame::variantName(g.Variant));
+      snprintf(z,128,"G-(%d)-(%d)-(%d)-(%s)",
+	       g.AmWhite ? 0 : 1,
+	       g.timecontrol.value[0], g.timecontrol.value[1], ChessGame::variantName(g.Variant));
       global.network->writeLine(z);
       MyProp = g;
       global.status->setText(_("Game proposal sent."),10);
@@ -155,20 +159,20 @@ void P2PProtocol::receiveString(char *netstring) {
 			Current.AmWhite ? BLACK : WHITE,
 			Current.Variant);
     if (Current.AmWhite)
-      sprintf(z,"%d. ... %s",MoveNumber,y);
+      snprintf(z,256,"%d. ... %s",MoveNumber,y);
     else
-      sprintf(z,"%d. %s",MoveNumber,y);
+      snprintf(z,256,"%d. %s",MoveNumber,y);
 
     if (MoveNumber == 1)
-      wc = bc = Current.STime;
+      wc = bc = 1000 * Current.timecontrol.value[0];
     else {
-      MyGame->incrementActiveClock(Current.SInc);
+      MyGame->incrementActiveClock(Current.timecontrol.value[1]);
       wc = bc = CLOCK_UNCHANGED;
     }
 
-    MyGame->updatePosition(cur, MoveNumber,
-			   cur.sidehint?0:1, 
-			   wc, bc, z, true);
+    MyGame->updatePosition2(cur, MoveNumber,
+			    cur.sidehint?0:1, 
+			    wc, bc, z, true);
 
     if (Current.AmWhite) ++MoveNumber;
     global.BoardList.front()->setCanMove(true);
@@ -188,7 +192,7 @@ void P2PProtocol::receiveString(char *netstring) {
     }
     p = ProtoGreet.getStarToken(1);
     memset(RemoteSoftware,0,128);
-    strncpy(RemoteSoftware,p,127);
+    g_strlcpy(RemoteSoftware,p,128);
 
     RemoteProtover = atoi(ProtoGreet.getStarToken(2));
     if (RemoteProtover < 1) {
@@ -197,8 +201,8 @@ void P2PProtocol::receiveString(char *netstring) {
       return;
     }
     
-    sprintf(z,"Remote software: %s (eboard/p2p v.%d)",
-	    RemoteSoftware,RemoteProtover);
+    snprintf(z,256,"Remote software: %s (eboard/p2p v.%d)",
+	     RemoteSoftware,RemoteProtover);
     global.output->append(z,global.Colors.TextBright,IM_NORMAL);
     IdReceived = true;
     sendId();
@@ -213,9 +217,9 @@ void P2PProtocol::receiveString(char *netstring) {
 
   if (NameGreet.match()) {
     memset(RemotePlayer,0,64);
-    strncpy(RemotePlayer,NameGreet.getStarToken(0),127);
-    sprintf(z,"Remote player name: %s",
-	    RemotePlayer);
+    g_strlcpy(RemotePlayer,NameGreet.getStarToken(0),64);
+    snprintf(z,256,"Remote player name: %s",
+	     RemotePlayer);
     global.output->append(z,global.Colors.TextBright,IM_NORMAL);
     sendId();
     return;
@@ -223,23 +227,25 @@ void P2PProtocol::receiveString(char *netstring) {
 
   if (GameProp.match()) {
     GameProposal g;
+    int a,b;
 
     g.AmWhite = (atoi(GameProp.getStarToken(0)) != 0);
-    g.STime   = atoi(GameProp.getStarToken(1));
-    g.SInc    = atoi(GameProp.getStarToken(2));
+    a = atoi(GameProp.getStarToken(1));
+    b = atoi(GameProp.getStarToken(2));
+    g.timecontrol.setIcs(a,b);
     g.Variant = ChessGame::variantFromName(GameProp.getStarToken(3));
 
     HisProp = g;
     GotProp = true;
     if (pad)
       pad->setProposal(g);
-    sprintf(z,_("Received a game proposal from %s."),RemotePlayer);
+    snprintf(z,256,_("Received a game proposal from %s."),RemotePlayer);
     global.output->append(z,global.Colors.TextBright,IM_NORMAL);
     return;
   }
 
   if (Accept.match()) {
-    sprintf(z,_("%s accepted your game proposal."),RemotePlayer);
+    snprintf(z,256,_("%s accepted your game proposal."),RemotePlayer);
     global.output->append(z,global.Colors.TextBright,IM_NORMAL);
     createGame(MyProp);
     return;
@@ -252,7 +258,7 @@ void P2PProtocol::receiveString(char *netstring) {
     } else {
       char z[128];
       pad->setDrawProp();
-      sprintf(z,_("%s offers a draw."),RemotePlayer);
+      snprintf(z,128,_("%s offers a draw."),RemotePlayer);
       global.output->append(z,global.Colors.TextBright,IM_PERSONAL);
       GotDrawProp = true;
       GotProp = false;
@@ -292,9 +298,9 @@ void P2PProtocol::receiveString(char *netstring) {
   }
 
   if (strlen(netstring) < 200)
-    sprintf(z,"Got garbage: [%s]",netstring);
+    snprintf(z,256,"Got garbage: [%s]",netstring);
   else
-    sprintf(z,"Got too much garbage.");
+    snprintf(z,256,"Got too much garbage.");
   global.output->append(z,global.Colors.Engine,IM_NORMAL);
 }
 
@@ -302,7 +308,7 @@ void P2PProtocol::sendUserInput(char *line) {
   int j;
 
   memset(tmpbuf,0,2048);
-  sprintf(tmpbuf,"C-%s> ",global.P2PName);
+  snprintf(tmpbuf,2048,"C-%s> ",global.P2PName);
   j = strlen(line);
   strncat(tmpbuf, line, 1900);
   if (global.network)
@@ -320,13 +326,12 @@ void P2PProtocol::createGame(GameProposal &g) {
   cg->source = GS_Other;
   cg->clock_regressive = 1;
   cg->GameNumber=global.nextFreeGameId(P2P_GAME+1);
-  strncpy(cg->PlayerName[g.AmWhite?0:1],global.P2PName,64);
-  strncpy(cg->PlayerName[g.AmWhite?1:0],RemotePlayer,64);
+  g_strlcpy(cg->PlayerName[g.AmWhite?0:1],global.P2PName,64);
+  g_strlcpy(cg->PlayerName[g.AmWhite?1:0],RemotePlayer,64);
   cg->MyColor = g.AmWhite ? WHITE : BLACK;
   cg->Rated = 0;
   cg->Variant = g.Variant;
-  cg->STime = g.STime;
-  cg->Inc = g.SInc;
+  cg->timecontrol = g.timecontrol;
   cg->AmPlaying = true;
 
   global.prependGame(cg);
@@ -340,13 +345,13 @@ void P2PProtocol::createGame(GameProposal &g) {
   global.BoardList.front()->setFlipped(!g.AmWhite);
 
   cg->acknowledgeInfo();
-  cg->fireWhiteClock(g.STime, g.STime);
+  cg->fireWhiteClock(g.timecontrol.value[0], g.timecontrol.value[1]);
 
   p = new Position();
-  cg->updatePosition(*p,1,p->sidehint ? 0:1,
-		     g.STime,
-		     g.STime,
-		     "0. startpos");
+  cg->updatePosition2(*p,1,p->sidehint ? 0:1,
+		      1000*g.timecontrol.value[0],
+		      1000*g.timecontrol.value[0],
+		      "0. startpos");
   delete p;
 
   global.status->setText(_("Game started!"),10);
@@ -382,21 +387,21 @@ void P2PProtocol::sendMove(int x1,int y1,int x2,int y2,int prom) {
   cur.moveCartesian(x1,y1,x2,y2,MyGame->Variant,true);
 
   if (Current.AmWhite)
-    sprintf(xsan,"%d. %s",MoveNumber,san);
+    snprintf(xsan,20,"%d. %s",MoveNumber,san);
   else
-    sprintf(xsan,"%d. ... %s",MoveNumber,san);
+    snprintf(xsan,20,"%d. ... %s",MoveNumber,san);
 
   if (MoveNumber == 1)
-    wc = bc = Current.STime;
+    wc = bc = 1000 * Current.timecontrol.value[0];
   else {
     wc = bc = CLOCK_UNCHANGED;
-    MyGame->incrementActiveClock(Current.SInc);
+    MyGame->incrementActiveClock(Current.timecontrol.value[1]);
   }
 
   if (!Current.AmWhite) ++MoveNumber;
 
-  MyGame->updatePosition(cur,MoveNumber,
-			 cur.sidehint ? 0:1, wc,bc, xsan);
+  MyGame->updatePosition2(cur,MoveNumber,
+			  cur.sidehint ? 0:1, wc,bc, xsan);
 
   /* -- */
 
@@ -419,26 +424,26 @@ void P2PProtocol::sendMove(int x1,int y1,int x2,int y2,int prom) {
     case KING:    move[5]='K'; break;
     }
   }
-  sprintf(xm,"M-(%s)",move);
+  snprintf(xm,12,"M-(%s)",move);
   global.network->writeLine(xm);
   global.BoardList.front()->setCanMove(false);
 
   /* check if move ends the game */
 
   if (cur.isStalemate(Current.AmWhite?BLACK:WHITE,Current.Variant)) {
-    sprintf(xm,"R-(D)-(S)");
+    snprintf(xm,12,"R-(D)-(S)");
     global.network->writeLine(xm);
     gameOver(DRAW, _("Stalemate"));
     return;
   } 
   if (cur.isNMDraw(Current.Variant)) {
-    sprintf(xm,"R-(D)-(N)");
+    snprintf(xm,12,"R-(D)-(N)");
     global.network->writeLine(xm);
     gameOver(DRAW, _("No material to mate"));
     return;
   }
   if (cur.isMate(Current.AmWhite?BLACK:WHITE,Current.Variant)) {
-    sprintf(xm,"R-(%c)-(M)", Current.AmWhite?'W':'B');
+    snprintf(xm,12,"R-(%c)-(M)", Current.AmWhite?'W':'B');
     global.network->writeLine(xm);
     gameOver(Current.AmWhite?WHITE_WIN:BLACK_WIN, _("Checkmate"));
     return;
@@ -453,13 +458,13 @@ void P2PProtocol::gameOver(GameResult gr, char *desc) {
   if (!MyGame) return;
   MyGame->endGame(desc, gr);
 
-  sprintf(z,_("Game over: %s"), desc);
+  snprintf(z,256,_("Game over: %s"), desc);
   global.output->append(z, global.Colors.TextBright,
 			IM_NORMAL);
 
   if (global.AppendPlayed) {
     if  (MyGame->savePGN(global.AppendFile,true)) {
-      sprintf(z,_("Game appended to %s"),global.AppendFile);
+      snprintf(z,256,_("Game appended to %s"),global.AppendFile);
       global.status->setText(z,10);
     }
   }
@@ -479,7 +484,7 @@ void P2PProtocol::gameOver(GameResult gr, char *desc) {
 void P2PProtocol::resign() {
   char z[64];
   if (MyGame) {
-    sprintf(z,"R-(%c)-(R)",Current.AmWhite?'B':'W');
+    snprintf(z,64,"R-(%c)-(R)",Current.AmWhite?'B':'W');
     global.network->writeLine(z);
     gameOver(Current.AmWhite?BLACK_WIN:WHITE_WIN,_("Player resigns"));
   }
@@ -493,7 +498,7 @@ void P2PProtocol::draw() {
   } else {
     global.network->writeLine("D-O");
     SentDrawProp = true;
-    strcpy(z,_("Draw offer sent."));
+    g_strlcpy(z,_("Draw offer sent."),128);
     global.status->setText(z,5);
   }
 }
@@ -505,7 +510,7 @@ void P2PProtocol::adjourn() {
 void P2PProtocol::abort() {
   char z[64];
   if (MyGame) {
-    sprintf(z,"R-(D)-(A)");
+    snprintf(z,64,"R-(D)-(A)");
     global.network->writeLine(z);
     gameOver(UNDEF,_("Game Aborted"));
   }
@@ -542,7 +547,8 @@ P2PPad::P2PPad(P2PProtocol *_proto) : NonModalDialog("Direct Connection") {
   bl[1] = new BoxedLabel(_("Initial time ([mm:]ss):"));
   bl[2] = new BoxedLabel(_("Increment (secs):"));
 
-  color   = new DropBox(_("White"),_("Black"),0);
+  color   = new DropBox(_("White"),
+			_("Black"),NULL);
 
   wtime   = gtk_entry_new_with_max_length(8);
   winc    = gtk_entry_new_with_max_length(4);
@@ -597,7 +603,7 @@ P2PPad::P2PPad(P2PProtocol *_proto) : NonModalDialog("Direct Connection") {
   gtk_signal_connect(GTK_OBJECT(wacc),"clicked",
 		     GTK_SIGNAL_FUNC(p2ppad_accept), (gpointer) this);
 
-  Gtk::show(wtime,winc,f,t,h,match, wacc, h2, wprop, v2, f2, v, 0);
+  Gtk::show(wtime,winc,f,t,h,match, wacc, h2, wprop, v2, f2, v, NULL);
 }
 
 P2PPad::~P2PPad(){
@@ -610,11 +616,13 @@ P2PPad::~P2PPad(){
 void P2PPad::setProposal(GameProposal &g) {
   char z[512];
 
-  sprintf(z,_("%s (white) vs. %s (black)\n%s\n%d:%.2d %d"),
-	  g.AmWhite   ? global.P2PName : proto->getRemotePlayer(),
-	 (!g.AmWhite) ? global.P2PName : proto->getRemotePlayer(),
-	  ChessGame::variantName(g.Variant),
-	  g.STime / 60, g.STime % 60, g.SInc);
+  snprintf(z,512,_("%s (white) vs. %s (black)\n%s\n%d:%.2d %d"),
+	   g.AmWhite   ? global.P2PName : proto->getRemotePlayer(),
+	   (!g.AmWhite) ? global.P2PName : proto->getRemotePlayer(),
+	   ChessGame::variantName(g.Variant),
+	   g.timecontrol.value[0] / 60,
+	   g.timecontrol.value[0] % 60,
+	   g.timecontrol.value[1]);
   gtk_widget_set_sensitive(wacc, TRUE);
   gtk_label_set_text(GTK_LABEL(wprop), z);
   gtk_widget_queue_resize(wprop);
@@ -624,7 +632,7 @@ void P2PPad::setProposal(GameProposal &g) {
 
 void P2PPad::setDrawProp() {
   char z[128];
-  sprintf(z,_("%s offers a draw."),proto->getRemotePlayer());
+  snprintf(z,128,_("%s offers a draw."),proto->getRemotePlayer());
   gtk_widget_set_sensitive(wacc, TRUE);
   gtk_label_set_text(GTK_LABEL(wprop), z);
   gtk_widget_queue_resize(wprop);
@@ -639,7 +647,7 @@ void P2PPad::resetDrawProp() {
 
 void P2PPad::clearProposal() {
   char z[64];
-  strcpy(z,_("No proposals left."));
+  g_strlcpy(z,_("No proposals left."),64);
   gtk_widget_set_sensitive(wacc, FALSE);
   gtk_label_set_text(GTK_LABEL(wprop), z);
   gtk_widget_queue_resize(wprop);
@@ -658,16 +666,17 @@ void p2ppad_propose(GtkWidget *w, gpointer data) {
   P2PPad *me = (P2PPad *) data;
   GameProposal g;
   char z[64];
-  int T;
+  int T,a,b;
   tstring t;
   string *p;
 
   t.set(gtk_entry_get_text(GTK_ENTRY(me->wtime)));
-  g.STime = 0;
+  a = 0;
   while((p=t.token(":"))!=0)
-    g.STime = (60*g.STime) + atoi(p->c_str());
+    a = (60*a) + atoi(p->c_str());
   
-  g.SInc = atoi(gtk_entry_get_text(GTK_ENTRY(me->winc)));  
+  b = atoi(gtk_entry_get_text(GTK_ENTRY(me->winc)));  
+  g.timecontrol.setIcs(a,b);
   g.Variant = REGULAR;
   g.AmWhite = (me->color->getSelection() == 0);
 
@@ -677,13 +686,12 @@ void p2ppad_propose(GtkWidget *w, gpointer data) {
 
 GameProposal::GameProposal() {
   AmWhite = false;
-  STime = SInc = 0;
+  timecontrol.mode = TC_NONE;
   Variant = REGULAR;
 }
 
 GameProposal GameProposal::operator=(GameProposal &g) {
-  AmWhite = g.AmWhite;
-  STime   = g.STime;
-  SInc    = g.SInc;
-  Variant = g.Variant;
+  AmWhite     = g.AmWhite;
+  timecontrol = g.timecontrol;
+  Variant     = g.Variant;
 }
