@@ -1,10 +1,10 @@
-/* $Id: text.h,v 1.22 2003/01/31 06:01:21 bergo Exp $ */
+/* $Id: text.h,v 1.25 2008/02/22 14:34:30 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -58,7 +58,7 @@ class TextFilter {
   TextFilter();
   ~TextFilter();
   
-  void set(char *t);
+  void set(const char *t);
   const char *getString();
 
   bool accept(char *textline);
@@ -162,6 +162,13 @@ class DetachedConsole : public WidgetProxy {
   friend void dc_new_console(GtkWidget *w,gpointer data);
 };
 
+gint detached_delete  (GtkWidget * widget, GdkEvent * event, gpointer data);
+void detached_destroy (GtkWidget * widget, gpointer data);
+int  dc_input_key_press (GtkWidget * wid, GdkEventKey * evt,
+				gpointer data);
+void dc_set_filter(GtkWidget *w,gpointer data);
+void dc_new_console(GtkWidget *w,gpointer data);
+
 class TextFilterDialog : public ModalDialog {
  public:
   TextFilterDialog(Text *target, GtkWidget *label2update);
@@ -172,16 +179,6 @@ class TextFilterDialog : public ModalDialog {
   friend void tfd_ok(GtkWidget *w, gpointer data);
 };
 
-class SearchTextDialog : public ModalDialog {
- public:
-
-  SearchTextDialog(Searchable *src);
-
- private:
-  GtkWidget *text;
-  Searchable *searchable;
-
-  friend void std_ok(GtkWidget *w, gpointer data);
-};
+void tfd_ok(GtkWidget *w, gpointer data);
 
 #endif
