@@ -1,4 +1,4 @@
-/* $Id: tstring.h,v 1.4 2002/04/24 01:16:33 bergo Exp $ */
+/* $Id: tstring.h,v 1.6 2003/02/05 19:57:54 bergo Exp $ */
 
 /*
 
@@ -49,7 +49,7 @@ class tstring {
 
   /* sets the working string and resets position to its start */
   void set(string &s);
-  void set(char *s);
+  void set(const char *s);
 
   /* keeps working string but resets position to the start */
   void reset();
@@ -62,6 +62,10 @@ class tstring {
      characters of t and assumed to be in the given base */
   int    tokenvalue(char *t, int base=10);
 
+  /* returns false if number is zero, true if not, defval if no
+     more tokens */
+  bool   tokenbool(char *t, bool defval);
+
  private:
   string ptoken, src;
   int    pos;
